var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { gql } from '@apollo/client';
export const check = (hash, client) => __awaiter(void 0, void 0, void 0, function* () {
    const fetch = () => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const result = yield client.query({ query: gql `query FETCH_FOR_CHECK {
      mp: mp { id item_id path_item_depth path_item_id root_id position_id by_position(order_by: { path_item_depth: asc }) { id item_id path_item_depth path_item_id root_id position_id } }
      nodes: links { from_id id to_id type_id in { from_id id to_id type_id } out { from_id id to_id type_id } }
    }` });
        return { nodes: ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.nodes) || [], mp: ((_b = result === null || result === void 0 ? void 0 : result.data) === null || _b === void 0 ? void 0 : _b.mp) || [] };
    });
    const { nodes, mp } = yield fetch();
    let valid = true;
    const invalid = (...args) => {
        valid = false;
        console.log(...args);
    };
    const nodesChecked = {};
    const markersChecked = {};
    const checkNode = (node) => {
        var _a, _b, _c;
        if (nodesChecked[node.id])
            return;
        else
            nodesChecked[node.id] = true;
        const isLink = !!((node === null || node === void 0 ? void 0 : node.from_id) && (node === null || node === void 0 ? void 0 : node.to_id));
        const isRoot = isLink ? false : !((_a = node === null || node === void 0 ? void 0 : node.in) === null || _a === void 0 ? void 0 : _a.length);
        const markers = mp.filter((m) => m.item_id === node.id);
        const positions = mp.filter((m) => m.item_id === node.id && m.path_item_id === node.id);
        console.log(`check #${node.id} ${isLink ? 'link' : 'node'} in${(_b = node === null || node === void 0 ? void 0 : node.in) === null || _b === void 0 ? void 0 : _b.length} out${(_c = node === null || node === void 0 ? void 0 : node.out) === null || _c === void 0 ? void 0 : _c.length}`, positions.map((pos) => {
            return `${pos.root_id} [${pos.by_position.map((m) => `${m.path_item_id}`).join(',')}]`;
        }));
        if (isRoot) {
            if (markers.length !== 1)
                invalid(`invalid node #${node.id} root but markers.length = ${markers.length}`);
        }
        if (!markers.length)
            invalid(`invalid node #${node.id} markers lost, markers.length = ${markers.length}`);
        positions.forEach((position) => {
            checkPosition(position);
        });
    };
    const checkPosition = (position) => {
        position.by_position.forEach((marker, i) => {
            markersChecked[marker.id] = true;
            if (marker.position_id != position.position_id)
                invalid(`invalid position ${position.root_id} [${position.by_position.map((m) => m.path_item_id).join(',')}] position_id not equal`);
            const node = nodes.find((n) => n.id === marker.path_item_id);
            if (!node)
                invalid(`invalid position ${position.root_id} [${position.by_position.map((m) => m.path_item_id).join(',')}] node lost #${marker.path_item_id}`);
        });
    };
    nodes.forEach((node) => {
        checkNode(node);
    });
    mp.forEach((marker) => {
        if (!markersChecked[marker.id])
            invalid(`invalid marker #${marker.id} of node #${marker.item_id} ${marker.root_id}[...${marker.path_item_id}...]`);
    });
    if (!valid)
        throw new Error('invalid');
});
//# sourceMappingURL=check.js.map