var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Box, Divider, Flex, IconButton, Image, Input, InputGroup, InputLeftElement, InputRightElement, Text } from '@chakra-ui/react';
import { useDebounceCallback } from "@react-hook/debounce";
import { AnimatePresence, motion, useAnimation } from 'framer-motion';
import React, { useEffect, useRef, useState } from "react";
import { IoAddOutline, IoPlayOutline } from 'react-icons/io5';
import { BsFillPauseFill } from 'react-icons/bs';
import { MdDelete } from 'react-icons/md';
import { CustomizableIcon } from "../icons-provider";
import { ModalWindow } from "../modal-window";
import { DockerWarning } from './docker-warning';
import axios from 'axios';
import { Loading } from '../loading-motion-bubble';
import { useLocalStorage } from 'usehooks-ts';
const delay = ms => new Promise(resolve => setTimeout(resolve, ms));
const parseUrl = (text) => {
    try {
        const url = new URL(text);
        const gqlPath = `${url.hostname}${url.port ? ':' + url.port : ''}${url.pathname === '/' ? '' : url.pathname}`;
        const gqlSsl = url.protocol == 'http:' ? false : true;
        return [gqlPath, gqlSsl];
    }
    catch (_a) {
        return ['', false];
    }
};
const ConnectorGrid = React.memo((_a) => {
    var { children } = _a, props = __rest(_a, ["children"]);
    return (_jsx(Box, Object.assign({ display: 'flex' }, props, { children: _jsx(AnimatePresence, { children: children }) })));
});
const terminalAnimation = {
    grow: {
        scale: 1,
        width: '100%',
        opacity: 1,
        transition: { duration: 1.1 }
    },
    shrink: {
        scale: 0,
        opacity: 0,
        width: '0px',
        transition: { duration: 1.1, delay: 0.2 }
    }
};
const displayAnimation = {
    display: {
        display: 'block',
        transition: { duration: 1 }
    },
    none: {
        display: 'none',
        transition: { duration: 0.1, delay: 1.3 }
    },
    initial: {
        display: 'none',
    }
};
const callEngine = ({ serverUrl, operation, terminal }) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    terminal === null || terminal === void 0 ? void 0 : terminal.resize(terminal.cols, terminal.rows);
    const r = yield axios({
        method: 'post',
        url: `${serverUrl}/api/deeplinks`,
        headers: {
            'Content-Type': 'application/json'
        },
        data: {
            operation
        }
    });
    if (terminal) {
        terminal === null || terminal === void 0 ? void 0 : terminal.writeln(JSON.stringify((_a = r.data) === null || _a === void 0 ? void 0 : _a.envs));
        terminal === null || terminal === void 0 ? void 0 : terminal.writeln((_b = r.data) === null || _b === void 0 ? void 0 : _b.engineStr);
        const strings = (_e = (_d = (_c = r === null || r === void 0 ? void 0 : r.data) === null || _c === void 0 ? void 0 : _c.result) === null || _d === void 0 ? void 0 : _d.stdout) === null || _e === void 0 ? void 0 : _e.split('\n');
        if ((_g = (_f = r === null || r === void 0 ? void 0 : r.data) === null || _f === void 0 ? void 0 : _f.result) === null || _g === void 0 ? void 0 : _g.stderr)
            terminal === null || terminal === void 0 ? void 0 : terminal.writeln((_j = (_h = r === null || r === void 0 ? void 0 : r.data) === null || _h === void 0 ? void 0 : _h.result) === null || _j === void 0 ? void 0 : _j.stderr);
        for (let i = 0; i < (strings === null || strings === void 0 ? void 0 : strings.length); i++)
            terminal === null || terminal === void 0 ? void 0 : terminal.writeln(strings[i]);
        terminal === null || terminal === void 0 ? void 0 : terminal.writeln('');
    }
    return r;
});
const TerminalConnect = React.memo(({ initializingState = undefined, setInitLocal, serverUrl, setGqlPath, setGqlSsl, setPortal, deeplinksGqlPath, deeplinksGqlSsl, }) => {
    const terminalBoxRef = useRef();
    const terminalRef = useRef();
    const control = useAnimation();
    useEffect(() => {
        (() => __awaiter(void 0, void 0, void 0, function* () {
            const { Terminal } = yield import("xterm");
            const terminal = new Terminal({
                cursorBlink: true,
                cursorStyle: 'block',
            });
            terminalRef.current = terminal;
            if (terminalBoxRef.current) {
                terminal === null || terminal === void 0 ? void 0 : terminal.open(terminalBoxRef.current);
                terminal === null || terminal === void 0 ? void 0 : terminal.writeln('Ininitialization has been started. Please wait, \x1B[1;3;31mit may take a while.\x1B[0m');
            }
        }))();
    }, []);
    useEffect(() => {
        if (initializingState == 'initializing') {
            control.start('grow');
            console.log('initializingState0', initializingState);
            setTimeout(() => __awaiter(void 0, void 0, void 0, function* () {
                var _a;
                console.log('initializingState1', initializingState);
                (_a = terminalRef === null || terminalRef === void 0 ? void 0 : terminalRef.current) === null || _a === void 0 ? void 0 : _a.resize(terminalRef.current.cols, terminalRef.current.rows);
                setTimeout(() => {
                    var _a;
                    (_a = terminalRef === null || terminalRef === void 0 ? void 0 : terminalRef.current) === null || _a === void 0 ? void 0 : _a.focus();
                }, 1500);
                const initResult = yield callEngine({ serverUrl, operation: 'init', terminal: terminalRef.current });
                console.log(initResult);
                const migrateResult = yield callEngine({ serverUrl, operation: 'migrate', terminal: terminalRef.current });
                console.log(migrateResult);
                const checkResult = yield callEngine({ serverUrl, operation: 'check', terminal: terminalRef.current });
                console.log(checkResult);
                console.log('initializingState2', initializingState);
                yield delay(2000);
                setInitLocal(InitializingState.launched);
                setGqlPath(deeplinksGqlPath);
                setGqlSsl(deeplinksGqlSsl);
                yield delay(2000);
                setPortal(false);
            }), 2000);
        }
        else if (initializingState == 'removing') {
            console.log('initializingState0-r', initializingState);
            control.start('grow');
            setTimeout(() => __awaiter(void 0, void 0, void 0, function* () {
                var _b;
                console.log('initializingState1-r', initializingState);
                (_b = terminalRef === null || terminalRef === void 0 ? void 0 : terminalRef.current) === null || _b === void 0 ? void 0 : _b.resize(terminalRef.current.cols, terminalRef.current.rows);
                setTimeout(() => {
                    var _a;
                    (_a = terminalRef === null || terminalRef === void 0 ? void 0 : terminalRef.current) === null || _a === void 0 ? void 0 : _a.focus();
                }, 1500);
                yield callEngine({ serverUrl, operation: 'reset', terminal: terminalRef.current });
                console.log('initializingState2-r', initializingState);
                yield delay(2000);
                setInitLocal(InitializingState.notInit);
                yield delay(1000);
                setGqlPath('');
                setGqlSsl(false);
            }), 2000);
        }
        else if (initializingState == 'launched' || initializingState == 'not init') {
            if (terminalRef.current)
                terminalRef.current.clear();
            control.start('shrink');
        }
    }, [control, initializingState]);
    return (_jsx(AnimatePresence, { children: _jsx(Box, { as: motion.div, overflow: 'auto', sx: { overscrollBehavior: 'contain' }, borderRadius: '5px', border: '1rem solid #141214', animate: control, initial: 'shrink', variants: terminalAnimation, exit: 'shrink', children: _jsx(Box, { ref: terminalBoxRef, w: '45rem', h: '25rem', sx: {
                    '& > *': {
                        height: '100%',
                    },
                } }) }, 432432) }));
});
const ButtonTextButton = React.memo(({ ComponentLeftIcon = IoPlayOutline, ariaLabelLeft = 'Add local route', ComponentRightIcon = MdDelete, ariaLabelRight = 'Remove local route', text = 'Initialized', rightButtonId = '', leftButtonId = '', onClickLeft, onClickRight, }) => {
    return (_jsxs(Flex, { width: '100%', justify: 'space-between', alignItems: 'center', children: [_jsx(IconButton, { variant: 'unstyled', id: leftButtonId, size: 'md', "aria-label": ariaLabelLeft, icon: _jsx(ComponentLeftIcon, { color: 'rgb(0, 128, 255)' }), onClick: onClickLeft }), _jsx(Text, { color: 'gray.400', fontSize: 'sm', as: 'kbd', mr: '0.125rem', children: text }), _jsx(IconButton, { variant: 'unstyled', id: rightButtonId, size: 'md', "aria-label": ariaLabelRight, onClick: onClickRight, icon: _jsx(ComponentRightIcon, { color: 'rgb(0, 128, 255)' }) })] }));
});
const inputArea = {
    open: {
        height: '4rem',
        transition: { duration: 0.5 }
    },
    close: {
        height: '0rem',
        transition: { delay: 0.5 }
    },
    initial: {
        height: '0rem',
        overflow: 'hidden',
        originY: 1
    }
};
const inputAnimation = {
    add: {
        opacity: 1,
        scaleY: 1,
        originY: 0,
        transition: {
            duration: 0.5,
            delay: 0.2
        }
    },
    hide: {
        opacity: 0,
        scaleY: 0,
        originY: 1,
        transition: {
            duration: 0.3,
            display: { delay: 0.7 }
        }
    }
};
const InputAnimation = React.memo(({ bgContainer = '#141214', addRemoteRout = false, valueRemoteRoute = '', onChangeValueRemoteRoute, setValueRemote, onDeleteValue, onStartRemoteRoute, key, gqlPath, gqlSsl, }) => {
    const control = useAnimation();
    const controlInput = useAnimation();
    useEffect(() => {
        if (!!addRemoteRout) {
            control.start('open');
            controlInput.start("add");
        }
        else {
            controlInput.start("hide");
            control.start('close');
        }
    }, [addRemoteRout]);
    let isActive = false;
    let isBroken = false;
    const [currentGqlPath, currentGqlSsl] = parseUrl(valueRemoteRoute);
    isActive = currentGqlPath === gqlPath && currentGqlSsl === gqlSsl;
    isBroken = !currentGqlPath;
    return (_jsx(Box, { as: motion.div, animate: control, initial: 'initial', exit: 'close', layout: true, variants: inputArea, bg: bgContainer, w: '100%', display: 'flex', alignItems: 'center', justifyContent: 'center', pl: 4, pr: 4, children: _jsxs(InputGroup, { size: 'md', as: motion.div, animate: controlInput, initial: 'hide', exit: 'hide', color: 'gray.500', variants: inputAnimation, children: [_jsx(InputLeftElement, { onClick: () => {
                        const [currentGqlPath, currentGqlSsl] = parseUrl(valueRemoteRoute);
                        if (currentGqlPath) {
                            onStartRemoteRoute();
                        }
                    }, children: _jsx(CustomizableIcon, { Component: IoPlayOutline, value: { color: isBroken ? 'rgb(255, 0, 0)' : 'rgb(0, 128, 255)' } }) }), _jsx(Input, { placeholder: 'rout', value: valueRemoteRoute, onChange: onChangeValueRemoteRoute, borderColor: isBroken ? 'rgb(255, 0, 0)' : (isActive ? 'rgb(0, 128, 255)' : 'rgb(255, 255, 255)') }), _jsx(InputRightElement, { onClick: onDeleteValue, children: _jsx(CustomizableIcon, { Component: MdDelete, value: { color: 'rgb(0, 128, 255)' } }) })] }) }, key));
});
const initArea = {
    initial: {
        scaleY: 1,
        originY: 1,
        opacity: 1,
        display: 'flex'
    },
    open: {
        scaleY: 1,
        opacity: 1,
    },
    close: {
        scaleY: 0,
        opacity: 0,
    },
    initializing: {
        scaleY: 0,
        originY: 1,
        opacity: 0,
    }
};
var InitializingState;
(function (InitializingState) {
    InitializingState["notInit"] = "not init";
    InitializingState["initializing"] = "initializing";
    InitializingState["initialized"] = "initialized";
    InitializingState["launched"] = "launched";
    InitializingState["removing"] = "removing";
})(InitializingState || (InitializingState = {}));
export const checkSystemStatus = (deeplinksUrl) => __awaiter(void 0, void 0, void 0, function* () {
    var _k, _l, _m;
    let status;
    let err;
    try {
        status = yield axios.post(`${deeplinksUrl}/gql`, { "query": "{ healthz { status } }" }, { validateStatus: status => true, timeout: 7000 });
    }
    catch (e) {
        err = e;
    }
    console.log('system status', JSON.stringify(status === null || status === void 0 ? void 0 : status.data));
    return { result: (_m = (_l = (_k = status === null || status === void 0 ? void 0 : status.data) === null || _k === void 0 ? void 0 : _k.data) === null || _l === void 0 ? void 0 : _l.healthz) === null || _m === void 0 ? void 0 : _m[0].status, error: err };
});
export const Connector = React.memo(({ portalOpen = true, setPortal, gqlPath, gqlSsl, serverUrl, deeplinksUrl, setGqlPath, setGqlSsl, }) => {
    const control = useAnimation();
    const controlNotInit = useAnimation();
    const controlInit = useAnimation();
    const controlInited = useAnimation();
    const controlLaunch = useAnimation();
    const controlRemoving = useAnimation();
    const [valueRemote, setValueRemote] = useState('');
    const [isExistDocker, setIsExistDocker] = useState(true);
    const [init, setInitLocal] = useState(InitializingState.notInit);
    const onChangeValueRemote = useDebounceCallback((value) => {
        setValueRemote(value);
    }, 500);
    const [deeplinksPath, deeplinksSsl] = parseUrl(deeplinksUrl);
    const deeplinksGqlPath = deeplinksPath + '/gql';
    const deeplinksGqlSsl = deeplinksSsl;
    const [remotesString, setRemotesString] = useLocalStorage('remote-routes', '[]');
    const remotes = JSON.parse(remotesString);
    const checkUrlStatus = (url) => __awaiter(void 0, void 0, void 0, function* () {
        let status;
        let err;
        try {
            status = yield axios.get(url, { validateStatus: status => true, timeout: 7000 });
        }
        catch (e) {
            err = e;
        }
        return { result: status === null || status === void 0 ? void 0 : status.data, error: err };
    });
    const add = () => {
        setRemotesString((remotesString) => JSON.stringify([
            ...JSON.parse(remotesString),
            { id: (Math.random() + 1).toString(36).substring(7), value: "" }
        ]));
    };
    const remove = (id) => {
        setRemotesString((remotesString) => JSON.stringify(JSON.parse(remotesString).filter((el) => el.id != id)));
    };
    const save = (id, value) => {
        setRemotesString((remotesString) => JSON.stringify(JSON.parse(remotesString).map((el) => (el.id === id ? Object.assign(Object.assign({}, el), { value }) : el))));
    };
    useEffect(() => {
        if (!!portalOpen) {
            control.start("grow");
        }
        else {
            control.start("shrink");
        }
    }, [control, portalOpen]);
    useEffect(() => {
        if (init === InitializingState.initializing) {
            controlNotInit.start('close');
            controlInit.start('open');
        }
        else if (init === InitializingState.initialized) {
            controlNotInit.start('initializing');
            controlInit.start('initializing');
            controlInited.start('open');
        }
        else if (init === InitializingState.launched) {
            controlNotInit.start('initializing');
            controlInit.start('initializing');
            controlInited.start('initializing');
            controlLaunch.start('open');
        }
        else if (init === InitializingState.removing) {
            controlNotInit.start('initializing');
            controlInit.start('initializing');
            controlInited.start('initializing');
            controlLaunch.start('initializing');
            controlRemoving.start("open");
        }
        else if (init === InitializingState.notInit) {
            controlNotInit.start('initializing');
            controlInit.start('initializing');
            controlInited.start('initializing');
            controlLaunch.start('initializing');
            controlRemoving.start("initializing");
            controlNotInit.start("open");
        }
    }, [init]);
    useEffect(() => {
        (() => __awaiter(void 0, void 0, void 0, function* () {
            const status = yield checkSystemStatus(deeplinksUrl);
            if (status.result !== undefined) {
                setInitLocal(InitializingState.notInit);
                yield delay(1000);
                setInitLocal(InitializingState.launched);
                if (!gqlPath) {
                    setGqlPath(deeplinksGqlPath);
                    setGqlSsl(deeplinksGqlSsl);
                }
            }
        }))();
    }, [portalOpen]);
    useEffect(() => {
        (() => __awaiter(void 0, void 0, void 0, function* () {
            var _a, _b, _c, _d, _e;
            const dockerStatus = yield callEngine({ serverUrl, operation: 'dock' });
            console.log('dockerStatusObj', dockerStatus);
            if (((_c = (_b = (_a = dockerStatus === null || dockerStatus === void 0 ? void 0 : dockerStatus.data) === null || _a === void 0 ? void 0 : _a.result) === null || _b === void 0 ? void 0 : _b.stdout) === null || _c === void 0 ? void 0 : _c[0]) !== '{')
                setIsExistDocker(false);
            const dockerComposeStatus = yield callEngine({ serverUrl, operation: 'compose' });
            console.log('dockerComposeStatusObj', dockerComposeStatus);
            if (!/^-?[a-z0-9.]+\r?\n?$/.test((_e = (_d = dockerComposeStatus === null || dockerComposeStatus === void 0 ? void 0 : dockerComposeStatus.data) === null || _d === void 0 ? void 0 : _d.result) === null || _e === void 0 ? void 0 : _e.stdout.toString()))
                setIsExistDocker(false);
        }))();
    }, []);
    return (_jsx(ModalWindow, { onClosePortal: () => {
            if (init == InitializingState.launched) {
                setPortal(false);
            }
        }, portalOpen: portalOpen, children: _jsxs(Box, { display: 'flex', flexDirection: 'column', alignItems: 'center', children: [_jsx(Box, { children: _jsx(Box, { boxSize: '5rem', mb: 4, children: _jsx(Image, { src: './logo_n.svg', alt: 'logo' }) }) }), _jsxs(ConnectorGrid, { alignItems: 'center', sx: {
                        '& > *:not(:last-of-type)': {
                            mr: init == InitializingState.notInit ? 0 : 4,
                        }
                    }, children: [_jsxs(Box, { display: 'flex', flexDir: 'column', alignItems: 'center', justifyContent: 'center', height: '100%', width: 'max-content', bg: '#141214', borderRadius: '5px', children: [_jsx(Box, { pt: 4, pl: 4, pr: 4, textAlign: 'left', w: '100%', children: _jsx(Text, { color: 'gray.400', fontSize: 'md', children: "Remote deep" }) }), _jsx(AnimatePresence, { children: remotes.map(rr => (_jsx(InputAnimation, { addRemoteRout: !!remotes, valueRemoteRoute: rr.value, onChangeValueRemoteRoute: (e) => save(rr.id, e.target.value), gqlPath: gqlPath, gqlSsl: gqlSsl, onDeleteValue: (e) => {
                                            if (gqlPath == rr.value) {
                                                setGqlPath('');
                                                setGqlSsl(false);
                                            }
                                            remove(rr.id);
                                        }, onStartRemoteRoute: () => __awaiter(void 0, void 0, void 0, function* () {
                                            try {
                                                const url = new URL(rr.value);
                                                const status = yield checkUrlStatus(url);
                                                if (status.result !== undefined && !status.error) {
                                                    setGqlPath(`${url.hostname}${url.port ? ':' + url.port : ''}${url.pathname}`);
                                                    setGqlSsl(url.protocol == 'http:' ? false : true);
                                                    setPortal(false);
                                                }
                                                else {
                                                    console.log('URL error', JSON.stringify(status));
                                                }
                                            }
                                            catch (e) {
                                                console.log('URL error', e);
                                            }
                                        }) }, rr.id))) }), _jsxs(Box, { pb: 4, pl: 4, pr: 4, width: '100%', children: [_jsx(IconButton, { as: motion.div, variant: 'unstyled', "aria-label": 'Add remote route', icon: _jsx(IoAddOutline, { color: 'rgb(0, 128, 255)' }), onTap: add }), _jsx(Divider, { mb: 4 }), _jsx(Text, { color: 'gray.400', fontSize: 'md', children: "Local deep" }), !isExistDocker ? _jsx(DockerWarning, {}) : null] }), _jsx(Box, { bg: '#141214', pl: 4, pb: 2, boxSizing: 'border-box', w: '100%', position: 'relative', borderBottomLeftRadius: '5px', borderBottomRightRadius: '5px', filter: 'auto', blur: isExistDocker === false ? '2px' : 0, children: _jsxs(AnimatePresence, { children: [_jsxs(Box, { w: '100%', minW: '19.75rem', h: '100%', display: 'flex', justifyContent: 'space-between', alignItems: 'center', as: motion.div, animate: controlNotInit, initial: 'initial', variants: initArea, onClick: () => {
                                                    setInitLocal(InitializingState.initializing);
                                                }, children: [_jsx(Text, { color: 'gray.400', fontSize: 'sm', as: 'kbd', children: "no initialized" }), _jsx(IconButton, { variant: 'unstyled', size: 'md', "aria-label": 'Add local route', id: 'startInitLocal', icon: _jsx(IoAddOutline, { color: 'rgb(0, 128, 255)' }) })] }, InitializingState.notInit), _jsx(Box, { w: '100%', minW: '19.75rem', h: '100%', display: 'flex', position: 'absolute', top: 0, left: 4, justifyContent: 'space-between', alignItems: 'center', as: motion.div, animate: controlInit, initial: 'initializing', variants: initArea, children: _jsx(Loading, { text: "Initializing", sxFlex: { pt: 2, pb: 2 }, sx: {
                                                        pb: '0.3125rem',
                                                    } }) }, InitializingState.initializing), _jsx(Box, { w: '100%', minW: '19.75rem', h: '100%', position: 'absolute', top: 0, left: 0, as: motion.div, animate: controlInited, initial: 'initializing', variants: initArea, children: _jsx(ButtonTextButton, { ariaLabelLeft: "go to launched deepcase", ariaLabelRight: "launched local deepcase", ComponentRightIcon: BsFillPauseFill }) }, InitializingState.initialized), _jsx(Box, { w: '100%', minW: '19.75rem', h: '100%', position: 'absolute', top: 0, left: 0, as: motion.div, animate: controlLaunch, initial: 'initializing', variants: initArea, children: _jsx(ButtonTextButton, { text: 'launched', ariaLabelLeft: "go to deepcase", ariaLabelRight: "delete local deepcase", onClickLeft: () => {
                                                        setGqlPath(deeplinksGqlPath);
                                                        setGqlSsl(deeplinksGqlSsl);
                                                        setPortal(false);
                                                    }, leftButtonId: "goToDeep", rightButtonId: "deleteLocalDeep", onClickRight: () => {
                                                        setInitLocal(InitializingState.removing);
                                                    } }) }, InitializingState.launched), _jsx(Box, { w: '100%', minW: '19.75rem', h: '100%', position: 'absolute', top: 0, left: 0, pl: 4, as: motion.div, animate: controlRemoving, initial: 'initializing', variants: initArea, children: _jsx(Loading, { text: "Removing", sxFlex: { pt: 2, pb: 2 }, sx: {
                                                        pb: '0.3125rem',
                                                        '& > *:not(:last-of-type)': {
                                                            mr: 1
                                                        }
                                                    } }) }, InitializingState.removing)] }) })] }, 1221), _jsx(TerminalConnect, { initializingState: init, setInitLocal: (state) => setInitLocal(state), serverUrl: serverUrl, setGqlPath: (path) => setGqlPath(path), setGqlSsl: (ssl) => setGqlSsl(ssl), setPortal: (state) => setPortal(state), deeplinksGqlPath: deeplinksGqlPath, deeplinksGqlSsl: deeplinksGqlSsl }, 21121)] })] }) }));
});
//# sourceMappingURL=connector.js.map