var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { Box } from '@chakra-ui/react';
import { useDeep, useDeepSubscription } from '@deep-foundation/deeplinks/imports/client';
import { useMinilinksFilter } from '@deep-foundation/deeplinks/imports/minilinks';
import { ClientHandlerRenderer, evalClientHandler } from '../client-handler';
import { useLocalStore } from '@deep-foundation/store/local';
import { useCallback, useEffect, useRef, useState } from 'react';
import { useCytoEditor } from './hooks';
import { EditorComponentView } from '../editor/editor-component-view';
import { EditorGrid } from '../editor/editor-grid';
import { EditorSwitcher } from '../editor/editor-switcher';
import { EditorTextArea } from '../editor/editor-textarea';
import json5 from 'json5';
import { useDebounceCallback } from '@react-hook/debounce';
import { CatchErrors } from '../react-errors';
import { CytoEditorHandlers } from './handlers';
export function useEditorValueSaver(tab) {
    const [values, setValues] = useLocalStore('editor-values', {});
    const tempValueRef = useRef();
    const valuesRef = useRef();
    valuesRef.current = values;
    const setValuesDebounced = useDebounceCallback((value) => {
        setValues(value);
    }, 500);
    return {
        valuesRef,
        tempValueRef,
        value: values[tab],
        setValue: useCallback((id, value) => {
            tempValueRef.current = Object.assign(Object.assign({}, valuesRef.current), { [id]: value });
            setValuesDebounced(Object.assign(Object.assign({}, valuesRef.current), { [id]: value }));
        }, []),
    };
}
export function CytoEditorPreview({ link, compact = false, }) {
    var _a, _b, _c;
    const [cytoEditor, setCytoEditor] = useCytoEditor();
    const onClose = useCallback(() => {
        setCytoEditor(false);
    }, []);
    const deep = useDeep();
    const linkId = link.id;
    const [currentLinkId, setCurrentLinkId] = useState(linkId);
    const { data: [currentLink = link] = [] } = useDeepSubscription({
        id: currentLinkId,
    }, { name: 'CytoEditorPreview' });
    const { tempValueRef, valuesRef, value, setValue, } = useEditorValueSaver(linkId);
    const generatedLink = useMinilinksFilter(deep.minilinks, (link) => {
        var _a, _b, _c;
        return ((_c = (_b = (_a = link === null || link === void 0 ? void 0 : link.outByType) === null || _a === void 0 ? void 0 : _a[deep.idLocal('@deep-foundation/core', 'GeneratedFrom')]) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.to_id) === linkId;
    }, (link, ml) => {
        var _a, _b, _c;
        return (_c = (_b = (_a = ml.byId[linkId]) === null || _a === void 0 ? void 0 : _a.inByType[deep.idLocal('@deep-foundation/core', 'GeneratedFrom')]) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.from;
    });
    useEffect(() => {
        var _a, _b;
        const value = ((_a = generatedLink === null || generatedLink === void 0 ? void 0 : generatedLink.value) === null || _a === void 0 ? void 0 : _a.value) || ((_b = link === null || link === void 0 ? void 0 : link.value) === null || _b === void 0 ? void 0 : _b.value);
        console.log('editor-preview', 'evalClientHandler', 'useEffect', value);
        if (!value) {
            return;
        }
        evalClientHandler({ value, deep }).then(({ data, error }) => {
            console.log('editor-preview', 'evalClientHandler', 'error', error);
            console.log('editor-preview', 'evalClientHandler', 'data', data);
            setComponent(() => data);
        });
    }, [(_a = link === null || link === void 0 ? void 0 : link.value) === null || _a === void 0 ? void 0 : _a.value, generatedLink]);
    const currentValue = ((_b = valuesRef === null || valuesRef === void 0 ? void 0 : valuesRef.current) === null || _b === void 0 ? void 0 : _b[linkId]) || '';
    const refEditor = useRef();
    const [rightArea, setRightArea] = useState('preview');
    const [generated, setGenerated] = useState('src');
    const [fillSize, setFillSize] = useState(false);
    const [viewSize, setViewSize] = useState({ width: 124, height: 123 });
    const [Component, setComponent] = useState({});
    const [switcher, setSwitch] = useState(true);
    const switchProps = switcher ? { left: 0 } : { right: 0 };
    return _jsxs(_Fragment, { children: [_jsx(EditorGrid, { heightEditorGrid: '100%', columns: switcher ? '15% 85%' : '85% 15%', editorTextAreaElement: _jsx(_Fragment, { children: [_jsx(Box, { children: _jsx(EditorTextArea, { refEditor: refEditor, value: currentValue, onChange: (value) => {
                                    setValue(linkId, value);
                                }, onSave: (savedValue) => __awaiter(this, void 0, void 0, function* () {
                                    var _d, _e;
                                    const value = ((_d = tempValueRef === null || tempValueRef === void 0 ? void 0 : tempValueRef.current) === null || _d === void 0 ? void 0 : _d[linkId]) || savedValue;
                                    const Value = yield deep.id({ in: { type_id: { _id: ['@deep-foundation/core', 'Value'] }, from: { typed: { id: { _eq: linkId } } } } });
                                    const table = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'strings' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'numbers' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'objects' : undefined;
                                    const type = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'string' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'number' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'object' : 'undefined';
                                    let _value;
                                    try {
                                        _value = table === 'strings' ? value : table === 'numbers' ? parseFloat(value) : table === 'objects' ? json5.parse(value) : undefined;
                                    }
                                    catch (error) { }
                                    if (!((_e = deep.minilinks.byId[linkId]) === null || _e === void 0 ? void 0 : _e.value)) {
                                        yield deep.insert({ link_id: linkId, value: _value }, {
                                            table: table,
                                        });
                                    }
                                    else if (type !== 'undefined') {
                                        yield deep.update({ link_id: { _eq: linkId } }, {
                                            value: _value,
                                        }, {
                                            table: `${type}s`,
                                        });
                                    }
                                    else {
                                    }
                                }) }) }, linkId)] }), editorRight: rightArea === 'handlers' && (_jsx(CytoEditorHandlers, { linkId: generated && generatedLink ? generatedLink === null || generatedLink === void 0 ? void 0 : generatedLink.id : linkId })) ||
                    rightArea === 'preview' && _jsx(Box, { pos: 'relative', height: '100%', children: [_jsx(EditorComponentView, { size: viewSize, onChangeSize: (viewSize) => setViewSize(viewSize), fillSize: fillSize, setFillSize: setFillSize, children: typeof (Component) === 'function' && [_jsx(CatchErrors, { errorRenderer: (error) => {
                                            console.log('EditorComponentView', 'errorRenderer', error);
                                            return _jsx("div", { children: JSON.stringify(error) });
                                        }, children: _jsx(ClientHandlerRenderer, { Component: Component, fillSize: fillSize, link: (_c = deep === null || deep === void 0 ? void 0 : deep.minilinks) === null || _c === void 0 ? void 0 : _c.byId[currentLink === null || currentLink === void 0 ? void 0 : currentLink.id] }) }, Component.toString())] }, currentLink === null || currentLink === void 0 ? void 0 : currentLink.id)] }), editorRightSwitch: _jsx(EditorSwitcher, { fillSize: fillSize, setFillSize: (newFillSize) => {
                        setFillSize(newFillSize);
                        if (!fillSize)
                            setViewSize({ width: 250, height: 250 });
                    }, currentLinkId: currentLinkId, setCurrentLinkId: (newCurrentLinkId) => {
                        setCurrentLinkId(newCurrentLinkId);
                    }, generated: generated, setGenerated: setGenerated, area: rightArea, setArea: (rightArea) => {
                        setRightArea(rightArea);
                    } }) }), _jsx(Box, Object.assign({ position: 'absolute' }, switchProps, { top: 0, height: '100%', width: '15%', bg: 'primary', opacity: 0.2, onClick: () => {
                    setSwitch(sw => !sw);
                } }))] });
}
//# sourceMappingURL=editor-preview.js.map