var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Box, IconButton, Modal, ModalContent, ModalOverlay, useColorMode } from '@chakra-ui/react';
import { useDeep, useDeepSubscription } from '@deep-foundation/deeplinks/imports/client';
import { useMinilinksFilter } from '@deep-foundation/deeplinks/imports/minilinks';
import { useLocalStore } from '@deep-foundation/store/local';
import { useDebounceCallback } from '@react-hook/debounce';
import json5 from 'json5';
import dynamic from 'next/dynamic';
import { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { VscClearAll } from 'react-icons/vsc';
import { ClientHandler } from '../client-handler';
import { EditorComponentView } from '../editor/editor-component-view';
import { EditorGrid } from '../editor/editor-grid';
import { ListLanguages } from '../editor/editor-lang-list';
import { EditorResults } from '../editor/editor-results';
import { EditorSwitcher } from '../editor/editor-switcher';
import { CloseButton, EditorTabs } from '../editor/editor-tabs';
import { EditorTextArea } from '../editor/editor-textarea';
import { CytoEditorHandlers } from './handlers';
import { useCytoEditor } from './hooks';
const MonacoEditor = dynamic(() => import('@monaco-editor/react').then(m => m.default), { ssr: false });
export function useEditorValueSaver(tab) {
    const [values, setValues] = useLocalStore('editor-values', {});
    const tempValueRef = useRef();
    const valuesRef = useRef();
    valuesRef.current = values;
    const setValuesDebounced = useDebounceCallback((value) => {
        setValues(value);
    }, 500);
    return {
        valuesRef,
        tempValueRef,
        value: values[tab],
        setValue: useCallback((id, value) => {
            tempValueRef.current = Object.assign(Object.assign({}, valuesRef.current), { [id]: value });
            setValuesDebounced(Object.assign(Object.assign({}, valuesRef.current), { [id]: value }));
        }, []),
    };
}
export function useEditorTabs() {
    const [tabs, setTabs] = useLocalStore('editor-tabs', []);
    const [tab, setTab] = useLocalStore('editor-tab', 0);
    const tabsRef = useRef();
    tabsRef.current = tabs;
    return {
        tabs,
        addTab: useCallback((tab) => {
            if (!tabsRef.current.find((t) => t.id === tab.id)) {
                setTabs([...tabsRef.current, tab]);
            }
        }, []),
        closeTab: useCallback((id) => {
            const newTabs = tabsRef.current.filter((tab) => tab.id !== id);
            setTabs(newTabs);
            if (newTabs.length)
                setTab(newTabs[newTabs.length - 1].id);
            else
                setTab(0);
        }, []),
        tabId: tab,
        tab: tabs.find((t) => +t.id === +tab),
        activeTab: useCallback((id) => {
            setTab(id);
        }, []),
        setTab: useCallback((tab) => {
            setTabs(tabsRef.current.map((t) => (t.id === tab.id ? tab : t)));
        }, []),
        setTabs,
    };
}
export function useFindClientHandlerByCode({ codeLinkId, }) {
    const deep = useDeep();
    const [hid, setHid] = useState();
    const prevCodeLinkId = useRef(codeLinkId);
    useEffect(() => {
        (() => __awaiter(this, void 0, void 0, function* () {
            if (codeLinkId === prevCodeLinkId.current)
                return;
            const { data: handlers } = yield deep.select({
                execution_provider_id: { _eq: deep.idLocal('@deep-foundation/core', 'JSExecutionProvider'), },
                isolation_provider_id: { _eq: deep.idLocal('@deep-foundation/core', 'ClientJSIsolationProvider'), },
                _or: [
                    { dist_id: { _eq: codeLinkId } },
                    { src_id: { _eq: codeLinkId } },
                ],
            }, { table: 'handlers', returning: 'handler_id dist_id src_id' });
            if (handlers === null || handlers === void 0 ? void 0 : handlers[0])
                setHid(handlers === null || handlers === void 0 ? void 0 : handlers[0]);
            prevCodeLinkId.current = codeLinkId;
        }))();
    }, [codeLinkId, hid]);
    return hid;
}
export function CytoEditor() {
    var _a, _b, _c, _d;
    const [cytoEditor, setCytoEditor] = useCytoEditor();
    const onClose = useCallback(() => {
        setCytoEditor(false);
    }, []);
    const deep = useDeep();
    const { tab, tabs, closeTab, setTab, activeTab, tabId, setTabs, } = useEditorTabs();
    const [currentLinkId, setCurrentLinkId] = useState((tab === null || tab === void 0 ? void 0 : tab.id) || 0);
    const { data: [currentLink = (_a = deep === null || deep === void 0 ? void 0 : deep.minilinks) === null || _a === void 0 ? void 0 : _a.byId[tab === null || tab === void 0 ? void 0 : tab.id]] = [] } = useDeepSubscription({
        id: currentLinkId || 0,
    });
    const onCloseAll = useCallback(() => {
        setTabs([]);
    }, []);
    const { tempValueRef, valuesRef, value, setValue, } = useEditorValueSaver(tabId);
    const hid = useFindClientHandlerByCode({
        codeLinkId: tabId,
    });
    const handlerId = hid === null || hid === void 0 ? void 0 : hid.handler_id;
    const generatedLink = useMinilinksFilter(deep.minilinks, (link) => {
        var _a, _b, _c;
        const filterResult = ((_c = (_b = (_a = link === null || link === void 0 ? void 0 : link.outByType) === null || _a === void 0 ? void 0 : _a[deep.idLocal('@deep-foundation/core', 'GeneratedFrom')]) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.to_id) === tabId;
        return filterResult;
    }, (link, ml) => {
        var _a, _b, _c;
        return (_c = (_b = (_a = ml === null || ml === void 0 ? void 0 : ml.byId[tabId]) === null || _a === void 0 ? void 0 : _a.inByType[deep.idLocal('@deep-foundation/core', 'GeneratedFrom')]) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.from;
    });
    const [currentLanguage, setCurrentLanguage] = useLocalStore('df-dc-editor-currentLanguage', 'plaintext');
    useEffect(() => {
        var _a, _b;
        (_a = refEditor.current) === null || _a === void 0 ? void 0 : _a.monaco.editor.setModelLanguage((_b = refEditor.current) === null || _b === void 0 ? void 0 : _b.monaco.editor.getModels()[0], currentLanguage);
    }, [currentLanguage]);
    const currentValue = ((_b = valuesRef === null || valuesRef === void 0 ? void 0 : valuesRef.current) === null || _b === void 0 ? void 0 : _b[tabId]) || typeof (tab === null || tab === void 0 ? void 0 : tab.initialValue) !== 'undefined' ? tab === null || tab === void 0 ? void 0 : tab.initialValue : '';
    const refEditor = useRef();
    const [rightArea, setRightArea] = useState('preview');
    const [fillSize, setFillSize] = useState(false);
    const [viewSize, setViewSize] = useState({ width: '50%', height: '100%' });
    const [editorMounted, setEditorMounted] = useState(false);
    const errorRenderer = useMemo(() => {
        return (error, reset) => {
            return _jsx("div", { children: json5.stringify(error) });
        };
    }, []);
    const focusEditor = useCallback(() => {
        var _a, _b;
        (_b = (_a = refEditor === null || refEditor === void 0 ? void 0 : refEditor.current) === null || _a === void 0 ? void 0 : _a.editor) === null || _b === void 0 ? void 0 : _b.focus();
    }, []);
    useEffect(() => {
        import('@monaco-editor/react').then(m => { });
    }, []);
    const languages = (_c = refEditor.current) === null || _c === void 0 ? void 0 : _c.monaco.languages.getLanguages();
    const validationTS = (_d = refEditor.current) === null || _d === void 0 ? void 0 : _d.monaco.languages.typescript.typescriptDefaults.setDiagnosticsOptions({
        noSemanticValidation: true,
        noSyntaxValidation: true,
    });
    const { colorMode } = useColorMode();
    return _jsx(_Fragment, { children: _jsxs(Modal, { isOpen: cytoEditor, onClose: onClose, size: 'full', onEsc: onClose, children: [_jsx(ModalOverlay, {}), _jsx(ModalContent, { sx: { height: '100vh', overflow: 'initial' }, children: _jsx(EditorGrid, { sash: true, editorTextAreaElement: _jsx(_Fragment, { children: [
                                _jsxs(Box, { sx: {
                                        pos: 'relative',
                                        height: '100%'
                                    }, children: [_jsx(EditorTextArea, { refEditor: refEditor, value: currentValue, defaultLanguage: currentLanguage, onChange: (value) => {
                                                setValue(tabId, value);
                                                setTab(Object.assign(Object.assign({}, tab), { saved: tab.initialValue === value }));
                                            }, onClose: () => {
                                                var _a;
                                                if (tabs.length === 1 && ((_a = tabs[0]) === null || _a === void 0 ? void 0 : _a.id) === tab.id)
                                                    onClose();
                                                closeTab(tabId);
                                                setValue(tabId, undefined);
                                                focusEditor();
                                            }, onSave: (savedValue) => __awaiter(this, void 0, void 0, function* () {
                                                var _e, _f;
                                                const value = ((_e = tempValueRef === null || tempValueRef === void 0 ? void 0 : tempValueRef.current) === null || _e === void 0 ? void 0 : _e[tabId]) || savedValue;
                                                const Value = yield deep.id({ in: { type_id: { _id: ['@deep-foundation/core', 'Value'] }, from: { typed: { id: { _eq: tab.id } } } } });
                                                const table = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'strings' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'numbers' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'objects' : undefined;
                                                const type = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'string' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'number' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'object' : 'undefined';
                                                let _value;
                                                try {
                                                    _value = table === 'strings' ? value : table === 'numbers' ? parseFloat(value) : table === 'objects' ? json5.parse(value) : undefined;
                                                }
                                                catch (error) {
                                                    console.log('error123', error);
                                                }
                                                if (!((_f = deep.minilinks.byId[tab.id]) === null || _f === void 0 ? void 0 : _f.value)) {
                                                    yield deep.insert({ link_id: tab.id, value: _value }, {
                                                        table: table,
                                                    });
                                                    setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: true }));
                                                }
                                                else if (type !== 'undefined') {
                                                    yield deep.update({ link_id: { _eq: tab.id } }, {
                                                        value: _value,
                                                    }, {
                                                        table: `${type}s`,
                                                    });
                                                    setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: true }));
                                                }
                                                else {
                                                    setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: false }));
                                                }
                                            }), onMount: () => setEditorMounted(true) }), _jsx(Box, { w: '100%', pos: 'absolute', bottom: '0', borderTopColor: 'borderColor', borderTopWidth: 'thin', p: '0.5rem', height: 'auto', bg: 'lightDark', children: _jsx(Box, { pos: 'relative', height: '100%', children: _jsx(ListLanguages, { languages: languages, currentLanguage: currentLanguage, setLanguage: (i) => {
                                                        if (i == 'typescript')
                                                            validationTS;
                                                        setCurrentLanguage(i);
                                                    } }) }) })] }, tabId)
                            ] }), editorTabsElement: _jsx(EditorTabs, { tabs: tabs.map((tab) => {
                                var _a, _b, _c, _d, _e;
                                return (Object.assign(Object.assign({}, tab), { title: `${tab.id} ${((_e = (_d = (_c = (_b = (_a = deep.minilinks.byId[tab.id]) === null || _a === void 0 ? void 0 : _a.inByType) === null || _b === void 0 ? void 0 : _b[deep.idLocal('@deep-foundation/core', 'Contain')]) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.value) === null || _e === void 0 ? void 0 : _e.value) || ''}`.trim(), active: tabId === tab.id }));
                            }), setTabs: (tabs) => setTabs(tabs), onClose: (tab) => {
                                var _a;
                                if (tabs.length === 1 && ((_a = tabs[0]) === null || _a === void 0 ? void 0 : _a.id) === tab.id)
                                    onClose();
                                closeTab(tab.id);
                                setValue(tabId, undefined);
                                focusEditor();
                            }, onClick: (tab) => {
                                activeTab(tab.id);
                                focusEditor();
                            }, onSaveTab: (savedValue) => __awaiter(this, void 0, void 0, function* () {
                                var _g, _h;
                                const value = ((_g = tempValueRef === null || tempValueRef === void 0 ? void 0 : tempValueRef.current) === null || _g === void 0 ? void 0 : _g[tabId]) || savedValue;
                                const Value = yield deep.id({ in: { type_id: { _id: ['@deep-foundation/core', 'Value'] }, from: { typed: { id: { _eq: tab.id } } } } });
                                const table = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'strings' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'numbers' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'objects' : undefined;
                                const type = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'string' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'number' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'object' : 'undefined';
                                let _value;
                                try {
                                    _value = table === 'strings' ? value : table === 'numbers' ? parseFloat(value) : table === 'objects' ? json5.parse(value) : undefined;
                                }
                                catch (error) { }
                                if (!((_h = deep.minilinks.byId[tab.id]) === null || _h === void 0 ? void 0 : _h.value)) {
                                    yield deep.insert({ link_id: tab.id, value: _value }, {
                                        table: table,
                                    });
                                    setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: true }));
                                }
                                else if (type !== 'undefined') {
                                    yield deep.update({ link_id: { _eq: tab.id } }, {
                                        value: _value,
                                    }, {
                                        table: `${type}s`,
                                    });
                                    setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: true }));
                                }
                                else {
                                    setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: false }));
                                }
                                console.log('onclick');
                            }) }), closeAllButtonElement: _jsx(IconButton, { icon: _jsx(VscClearAll, {}), onClick: onCloseAll, "aria-label": 'Close all tabs' }), closeButtonElement: _jsx(CloseButton, { onClick: onClose }), editorRight: rightArea === 'handlers' && (_jsx(CytoEditorHandlers, { linkId: tab === null || tab === void 0 ? void 0 : tab.id, handleredableIds: generatedLink ? [generatedLink === null || generatedLink === void 0 ? void 0 : generatedLink.id, tab.id] : [tab === null || tab === void 0 ? void 0 : tab.id] })) ||
                            rightArea === 'preview' && _jsx(Box, { pos: 'relative', sx: {
                                    backgroundColor: 'editorPreviewBackground',
                                    backgroundImage: `linear-gradient(-90deg, ${'editorPreviewBackgroundGrid'} 1px, transparent 1px), linear-gradient(0deg, ${'editorPreviewBackgroundGrid'} 1px, transparent 1px), linear-gradient(transparent 0px, ${'editorPreviewBackground'} 1px, ${'editorPreviewBackground'} 20px, transparent 20px), linear-gradient(-90deg, ${'editorPreviewBackgroundGrid'} 1px, transparent 1px), linear-gradient(-90deg, transparent 0px, ${'editorPreviewBackground'} 1px, ${'editorPreviewBackground'} 20px, transparent 20px), linear-gradient(0deg, ${'editorPreviewBackgroundGrid'} 1px, transparent 1px)`,
                                    backgroundSize: '20px 20px, 20px 20px, 20px 20px, 20px 20px, 20px 20px, 20px 20px',
                                }, children: [_jsx(EditorComponentView, { size: viewSize, onChangeSize: (viewSize) => setViewSize(viewSize), fillSize: fillSize, setFillSize: setFillSize, children: handlerId && [_jsx(ClientHandler, { handlerId: handlerId, fillSize: fillSize, linkId: currentLink === null || currentLink === void 0 ? void 0 : currentLink.id, ml: deep.minilinks }, handlerId)] }, `${(currentLink === null || currentLink === void 0 ? void 0 : currentLink.id) || 0}-${tabId}-${handlerId}`)] }) ||
                            rightArea === 'results' && _jsx(EditorResults, { id: tab.id }), editorRightSwitch: _jsx(EditorSwitcher, { fillSize: fillSize, setFillSize: (newFillSize) => {
                                setFillSize(newFillSize);
                                if (!fillSize)
                                    setViewSize({ width: 250, height: 250 });
                            }, currentLinkId: currentLinkId, setCurrentLinkId: (newCurrentLinkId) => {
                                setCurrentLinkId(newCurrentLinkId);
                            }, area: rightArea, setArea: (rightArea) => {
                                setRightArea(rightArea);
                            } }) }) })] }) });
}
//# sourceMappingURL=editor.js.map