var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { ChevronDownIcon } from '@chakra-ui/icons';
import { Accordion, AccordionItem, AccordionButton, AccordionIcon, AccordionPanel, Box, Flex, Text, Button, Spacer, useColorMode, InputGroup, Input, InputRightElement, Tag, TagCloseButton, TagLabel, Menu, MenuButton, MenuItem, MenuList, IconButton } from '@chakra-ui/react';
import { useDeep, useDeepSubscription } from '@deep-foundation/deeplinks/imports/client';
import React, { useCallback, useRef, useState } from 'react';
import { useChackraColor } from '../get-color';
import { useContainer } from '../hooks';
import { VscCheck, VscDiffAdded } from 'react-icons/vsc';
export const CytoEditorHandlersSupportHandle = React.memo(function CytoEditorHandlersSupport({ support, handler, handle, }) {
    var _a, _b, _c, _d;
    const deep = useDeep();
    const gray900 = useChackraColor('gray.900');
    const white = useChackraColor('white');
    const { colorMode } = useColorMode();
    const [value, setValue] = useState('');
    const [adding, setAdding] = useState(false);
    const mouseoutRef = useRef();
    const onLeave = (e) => {
        clearTimeout(mouseoutRef.current);
        if (document.activeElement !== e.target)
            mouseoutRef.current = setTimeout(() => {
                setAdding(false);
                setInserting(false);
            }, 1000);
    };
    const HandleName = (_d = (_c = (_b = (_a = handle === null || handle === void 0 ? void 0 : handle.inByType) === null || _a === void 0 ? void 0 : _a[deep.idLocal('@deep-foundation/core', 'Contain')]) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.value) === null || _d === void 0 ? void 0 : _d.value;
    const [inserting, setInserting] = useState(false);
    const [deleting, setDeleting] = useState(false);
    const [container] = useContainer();
    const onInsert = useCallback(() => __awaiter(this, void 0, void 0, function* () {
        setInserting(true);
        yield deep.insert({
            type_id: handle.id,
            from_id: +value,
            to_id: handler.id,
            in: { data: {
                    type_id: deep.idLocal('@deep-foundation/core', 'Contain'),
                    from_id: container,
                } }
        });
        setInserting(false);
        setValue('');
    }), [value, container]);
    const onDelete = useCallback((handleId) => __awaiter(this, void 0, void 0, function* () {
        const link = deep.minilinks.byId[handleId];
        if (confirm(`Delete ${handleId} (${link.type_id}) = ${link.from_id} > ${link.to_id}?`)) {
            yield deep.delete(handleId);
        }
    }), []);
    const { data: handles } = useDeepSubscription({
        type_id: handle === null || handle === void 0 ? void 0 : handle.id,
        to_id: handler === null || handler === void 0 ? void 0 : handler.id,
    });
    const { data: ports } = useDeepSubscription({
        type_id: deep.idLocal('@deep-foundation/core', 'Port'),
    });
    const isLink = ['HandleInsert', 'HandleUpdate', 'HandleDelete'].includes(HandleName);
    const isRoute = HandleName === 'HandleRoute';
    const isPort = HandleName === 'HandlePort';
    const isShedule = HandleName === 'HandleShedule';
    const isClient = HandleName === 'HandleClient';
    const Handle = useCallback((isLink
        ? (h) => {
            return _jsxs(Tag, { size: 'sm', borderRadius: 'full', variant: 'solid', children: [_jsx(TagLabel, { children: h.from_id }), _jsx(TagCloseButton, { onClick: () => onDelete(h.id) })] });
        }
        : (h) => {
            return _jsxs(Tag, { size: 'sm', borderRadius: 'full', variant: 'solid', children: [_jsx(TagLabel, { children: h.from_id }), _jsx(TagCloseButton, { onClick: () => onDelete(h.id) })] });
        }), [deleting]);
    const Form = useCallback((isLink || isClient
        ? ({ value, onInsert }) => {
            return _jsxs(InputGroup, { position: 'absolute', w: '100%', size: 'md', h: '100%', left: 0, top: 0, borderWidth: '1px', borderRadius: 'lg', bgColor: 'handlersInput', onMouseMove: () => {
                    clearTimeout(mouseoutRef.current);
                }, children: [_jsx(Input, { autoFocus: true, h: '100%', type: 'number', placeholder: 'id', onMouseLeave: onLeave, onMouseOver: () => {
                            clearTimeout(mouseoutRef.current);
                            !adding && setAdding(true);
                        }, onBlur: onLeave, value: value, onChange: (e) => {
                            setValue(e.target.value);
                            console.log('value', value);
                        } }), value !== '' ? _jsx(InputRightElement, { h: '100%', children: _jsx(IconButton, { size: 'sm', variant: 'ghost', onClick: onInsert, isLoading: inserting, isRound: true, "aria-label": 'save and add id', icon: _jsx(VscCheck, {}) }) }) : null] });
        }
        : isPort
            ? ({ value, onInsert }) => {
                return _jsxs(InputGroup, { position: 'absolute', w: '100%', h: '100%', left: 0, top: 0, borderWidth: '1px', borderRadius: 'lg', bgColor: 'handlersInput', onMouseMove: () => {
                        clearTimeout(mouseoutRef.current);
                    }, children: [_jsxs(Menu, { children: [_jsx(MenuButton, { as: Button, rightIcon: _jsx(ChevronDownIcon, {}), h: '100%', w: `calc(100% - 60px)`, children: "Port" }), _jsxs(MenuList, { children: [_jsx(MenuItem, { children: "+" }), ports.map(port => (_jsx(MenuItem, { children: port.id }, port.id)))] })] }), value !== '' ? _jsx(InputRightElement, { h: '100%', children: _jsx(IconButton, { size: 'sm', variant: 'ghost', onClick: onInsert, isLoading: inserting, isRound: true, "aria-label": 'save and add id', icon: _jsx(VscCheck, {}) }) }) : null] });
            }
            : ({ value, onInsert }) => {
                return _jsxs(InputGroup, { position: 'absolute', w: '100%', h: '100%', left: 0, top: 0, borderWidth: '1px', borderRadius: 'lg', bgColor: 'handlersInput', onMouseMove: () => {
                        clearTimeout(mouseoutRef.current);
                    }, children: [_jsx(Input, { autoFocus: true, h: '100%', type: 'number', placeholder: 'id', onMouseLeave: onLeave, onMouseOver: () => {
                                clearTimeout(mouseoutRef.current);
                                !adding && setAdding(true);
                            }, onBlur: onLeave, value: value, onChange: (e) => {
                                setValue(e.target.value);
                                console.log('value', value);
                            } }), value !== '' ? _jsx(InputRightElement, { h: '100%', children: _jsx(IconButton, { size: 'sm', variant: 'ghost', onClick: onInsert, isLoading: inserting, isRound: true, "aria-label": 'save and add id', icon: _jsx(VscCheck, {}) }) }) : null] });
            }), []);
    return _jsx(Box, { width: '100%', children: _jsxs(Box, { borderWidth: '1px', borderRadius: 'lg', display: 'flex', flexDirection: 'column', children: [_jsx(Box, { p: '0.5rem', children: _jsxs(Flex, { position: 'relative', p: '0.3rem', borderWidth: 'thin', borderRadius: 'lg', children: [_jsx(Text, { p: 1, children: HandleName }), _jsx(Spacer, {}), _jsx(IconButton, { icon: _jsx(VscDiffAdded, {}), "aria-label": 'insert handle', size: 'sm', variant: 'ghost', isRound: true, onClick: () => setAdding(true), disabled: !(isLink || isPort || isClient) }), adding && _jsx(Form, { value: value, onInsert: onInsert })] }) }), !!(handles === null || handles === void 0 ? void 0 : handles.length) && _jsx(_Fragment, { children: _jsx(Box, { pl: '0.5rem', pb: '0.2rem', float: 'left', sx: {
                            '& > *:not(:last-of-type)': {
                                marginRight: '0.5rem'
                            }
                        }, children: handles.map(Handle) }) })] }) });
});
export const CytoEditorHandlersSupport = React.memo(function CytoEditorHandlersSupport({ support, linkId, handleredableIds, }) {
    var _a, _b, _c;
    const deep = useDeep();
    const { data: handlers } = useDeepSubscription({
        type_id: deep.idLocal('@deep-foundation/core', 'Handler'),
        from_id: support.id,
        to_id: { _in: handleredableIds },
    });
    const [container] = useContainer();
    const onInsertHandler = useCallback(() => __awaiter(this, void 0, void 0, function* () {
        yield deep.insert({
            to_id: linkId,
            from_id: support.id,
            type_id: deep.idLocal('@deep-foundation/core', 'Handler'),
            in: { data: {
                    type_id: deep.idLocal('@deep-foundation/core', 'Contain'),
                    from_id: container,
                } }
        });
    }), [container]);
    const onDeleteHandler = useCallback((handlerId) => __awaiter(this, void 0, void 0, function* () {
        yield deep.delete(handlerId);
    }), []);
    return _jsxs(AccordionItem, { children: [_jsxs(AccordionButton, { children: [_jsxs(Flex, { w: '100%', alignItems: 'center', children: [_jsxs(Box, { flex: '1', textAlign: 'left', children: ["Support: ", support.id, " ", (_c = (_b = (_a = support.inByType[deep.idLocal('@deep-foundation/core', 'Contain')]) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.value] }), _jsx(Spacer, {}), _jsx(IconButton, { "aria-label": 'insert handler', isRound: true, size: 'sm', variant: 'ghost', onClick: (e) => {
                                    e.stopPropagation();
                                    onInsertHandler();
                                }, icon: _jsx(VscDiffAdded, {}) })] }), _jsx(AccordionIcon, {})] }), _jsx(AccordionPanel, { pt: '0.5rem', pb: '0.5rem', children: _jsx(Box, { display: 'grid', gridTemplateColumns: 'repeat(2, 1fr)', gap: '0.5rem', children: handlers === null || handlers === void 0 ? void 0 : handlers.map((h) => {
                        var _a, _b, _c, _d;
                        return (_jsxs(Box, { borderWidth: '2px', borderRadius: 'lg', borderStyle: 'dashed', p: 2, children: [_jsxs(Box, { marginBottom: '0.5rem', children: [_jsx(Text, { children: "Handler:" }), " ", _jsxs(Tag, { size: 'sm', borderRadius: 'full', variant: 'solid', children: [_jsx(TagLabel, { children: h.id }), _jsx(TagCloseButton, { onClick: () => onDeleteHandler(h.id) })] }), " ", ((_c = (_b = (_a = h.inByType[deep.idLocal('@deep-foundation/core', 'Contain')]) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.value) || ''] }), _jsx(Box, { width: '100%', sx: {
                                        '& > *:not(:last-of-type)': {
                                            mb: '0.5rem',
                                        }
                                    }, children: (_d = support === null || support === void 0 ? void 0 : support.outByType[deep.idLocal('@deep-foundation/core', 'SupportsCompatable')]) === null || _d === void 0 ? void 0 : _d.map(({ to: handle }) => (_jsx(CytoEditorHandlersSupportHandle, { support: support, handler: h, handle: handle }))) })] }));
                    }) }) })] });
});
export const CytoEditorHandlers = React.memo(function CytoEditorHandlers({ linkId, handleredableIds, }) {
    const deep = useDeep();
    useDeepSubscription({
        down: {
            link: {
                type_id: { _in: [
                        deep.idLocal('@deep-foundation/core', 'Supports'),
                        deep.idLocal('@deep-foundation/core', 'SupportsCompatable'),
                        deep.idLocal('@deep-foundation/core', 'HandleOperation'),
                    ] },
            },
        },
    });
    useDeepSubscription({
        up: {
            parent: {
                type_id: { _in: [
                        deep.idLocal('@deep-foundation/core', 'Supports'),
                        deep.idLocal('@deep-foundation/core', 'SupportsCompatable'),
                        deep.idLocal('@deep-foundation/core', 'HandleOperation'),
                    ] },
            },
        },
    });
    useDeepSubscription({
        in: { type_id: deep.idLocal('@deep-foundation/core', 'SupportsCompatable') },
    });
    const supports = deep.useMinilinksSubscription({
        type_id: deep.idLocal('@deep-foundation/core', 'Supports'),
    });
    return _jsx(_Fragment, { children: _jsx(Box, { position: "relative", children: _jsx(Box, { h: '100%', w: '100%', overflowY: "scroll", position: "absolute", children: _jsx(Accordion, { allowMultiple: true, children: supports === null || supports === void 0 ? void 0 : supports.map((support) => {
                        return _jsx(CytoEditorHandlersSupport, { support: support, linkId: linkId, handleredableIds: handleredableIds });
                    }) }) }) }) });
});
//# sourceMappingURL=handlers.js.map