var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import _flatten from 'lodash/flatten';
import _isEqual from 'lodash/isEqual';
export function initializeTraveler(ncy, deepRef, spaceIdRef) {
    let Traveler;
    (() => __awaiter(this, void 0, void 0, function* () {
        const deep = deepRef.current;
        try {
            Traveler = yield deep.id('@deep-foundation/deepcase', 'Traveler');
        }
        catch (e) {
            console.log(e, deep.linkId, deep);
        }
    }))();
    const insertTraveler = (query, linkId) => __awaiter(this, void 0, void 0, function* () {
        const spaceId = spaceIdRef.current;
        const deep = deepRef.current;
        yield deep.insert({
            type_id: Traveler,
            from_id: linkId,
            in: { data: {
                    type_id: deep.idLocal('@deep-foundation/core', 'Contain'),
                    from_id: spaceId,
                } },
            to: { data: {
                    type_id: deep.idLocal('@deep-foundation/core', 'Query'),
                    object: { data: { value: query } },
                    in: { data: [
                            {
                                type_id: deep.idLocal('@deep-foundation/core', 'Contain'),
                                from_id: spaceId,
                            },
                            {
                                type_id: deep.idLocal('@deep-foundation/core', 'Active'),
                                from_id: spaceId,
                                in: { data: {
                                        type_id: deep.idLocal('@deep-foundation/core', 'Contain'),
                                        from_id: spaceId,
                                    } },
                            },
                        ] },
                } },
        });
    });
    const deleteTraveler = (query, linkId) => __awaiter(this, void 0, void 0, function* () {
        const deep = deepRef.current;
        const travelers = findTravlers(query, linkId);
        if (!(travelers === null || travelers === void 0 ? void 0 : travelers.length))
            return;
        const queries = travelers.map(t => t.to).filter(l => !!l);
        const actives = queries.map(q => { var _a, _b; return (_b = (_a = q.inByType) === null || _a === void 0 ? void 0 : _a[deep.idLocal('@deep-foundation/core', 'Active')]) === null || _b === void 0 ? void 0 : _b[0]; }).filter(l => !!l);
        yield deepRef.current.delete([
            ...travelers.map(l => l.id),
            ..._flatten(travelers.map(l => l.inByType[deep.idLocal('@deep-foundation/core', 'Contain')].map(l => l.id))),
            ...queries.map(l => l.id),
            ..._flatten(queries.map(l => l.inByType[deep.idLocal('@deep-foundation/core', 'Contain')].map(l => l.id))),
            ...actives.map(l => l.id),
            ..._flatten(actives.map(l => l.inByType[deep.idLocal('@deep-foundation/core', 'Contain')].map(l => l.id))),
        ]);
    });
    const switchTraveler = (query, linkId) => __awaiter(this, void 0, void 0, function* () {
        const travelers = findTravlers(query, linkId);
        if (travelers === null || travelers === void 0 ? void 0 : travelers.length) {
            yield deleteTraveler(query, linkId);
        }
        else {
            yield insertTraveler(query, linkId);
        }
    });
    const findTravlers = (query, linkId) => {
        var _a, _b, _c, _d, _e, _f, _g;
        const spaceId = spaceIdRef.current;
        const deep = deepRef.current;
        if (!Traveler)
            return [];
        if (query)
            return (_d = (_c = (_b = (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.byId[linkId]) === null || _b === void 0 ? void 0 : _b.outByType) === null || _c === void 0 ? void 0 : _c[Traveler]) === null || _d === void 0 ? void 0 : _d.filter(t => { var _a, _b; return _isEqual((_b = (_a = t === null || t === void 0 ? void 0 : t.to) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.value, query); });
        else
            return (_g = (_f = (_e = deep.minilinks) === null || _e === void 0 ? void 0 : _e.byId[linkId]) === null || _f === void 0 ? void 0 : _f.outByType) === null || _g === void 0 ? void 0 : _g[Traveler];
    };
    const nodeMenuTraveler = ncy.cxtmenu({
        selector: '.link-node',
        outsideMenuCancel: 10,
        openMenuEvents: 'ctxmenu-travelerMenu-open',
        closeMenuEvents: 'ctxmenu-travelerMenu-close',
        commands: [
            ((query) => ({
                content: (ele) => { var _a, _b, _c; return `types ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                contentStyle: { fontSize: '0.9rem', transform: 'rotate(65deg)' },
                select: function (ele) {
                    var _a;
                    return __awaiter(this, void 0, void 0, function* () {
                        const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                        if (id) {
                            yield switchTraveler(query(id), id);
                        }
                    });
                }
            }))((id) => ({
                down: {
                    tree_id: { _eq: deepRef.current.idLocal('@deep-foundation/core', 'typesTree') },
                    link_id: { _eq: deepRef.current.minilinks.byId[id].type_id },
                },
            })),
            ((query) => ({
                content: (ele) => { var _a, _b, _c; return `out ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                select: function (ele) {
                    var _a;
                    return __awaiter(this, void 0, void 0, function* () {
                        const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                        if (id) {
                            generateMenuByTypes(ele, query(id));
                        }
                    });
                }
            }))((id) => ({
                from_id: deepRef.current.minilinks.byId[id].id,
            })),
            ((query) => ({
                content: (ele) => { var _a, _b, _c; return `in ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                select: function (ele) {
                    var _a;
                    return __awaiter(this, void 0, void 0, function* () {
                        const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                        if (id) {
                            generateMenuByTypes(ele, query(id));
                        }
                    });
                }
            }))((id) => ({
                to_id: deepRef.current.minilinks.byId[id].id,
            })),
            ((query) => ({
                content: (ele) => { var _a, _b, _c; return `up ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                contentStyle: { fontSize: '0.9rem', transform: 'rotate(-65deg)' },
                select: function (ele) {
                    var _a;
                    return __awaiter(this, void 0, void 0, function* () {
                        const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                        if (id) {
                            generateMenuByTrees(ele, query(id), 'down');
                        }
                    });
                }
            }))((id) => ({
                down: {
                    link_id: deepRef.current.minilinks.byId[id].id,
                },
            })),
            ((query) => ({
                content: (ele) => { var _a, _b, _c; return `down ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                contentStyle: { fontSize: '0.9rem', transform: 'rotate(65deg)' },
                select: function (ele) {
                    var _a;
                    return __awaiter(this, void 0, void 0, function* () {
                        const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                        if (id) {
                            generateMenuByTrees(ele, query(id), 'up');
                        }
                    });
                }
            }))((id) => ({
                up: {
                    parent_id: deepRef.current.minilinks.byId[id].id,
                },
            })),
            ((query) => ({
                content: (ele) => { var _a, _b, _c; return `to ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                select: function (ele) {
                    var _a;
                    return __awaiter(this, void 0, void 0, function* () {
                        const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                        if (id) {
                            yield switchTraveler(query(id), id);
                        }
                    });
                }
            }))((id) => ({
                id: deepRef.current.minilinks.byId[id].to_id,
            })),
            ((query) => ({
                content: (ele) => { var _a, _b, _c; return `from ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                select: function (ele) {
                    var _a;
                    return __awaiter(this, void 0, void 0, function* () {
                        const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                        if (id) {
                            yield switchTraveler(query(id), id);
                        }
                    });
                }
            }))((id) => ({
                id: deepRef.current.minilinks.byId[id].from_id,
            })),
            ((query) => ({
                content: (ele) => { var _a, _b, _c; return `typed ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                contentStyle: { fontSize: '0.9rem', transform: 'rotate(-65deg)' },
                select: function (ele) {
                    var _a;
                    return __awaiter(this, void 0, void 0, function* () {
                        const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                        if (id) {
                            yield switchTraveler(query(id), id);
                        }
                    });
                }
            }))((id) => ({
                type_id: deepRef.current.minilinks.byId[id].id,
            })),
        ],
    });
    const generateMenuByTypes = (ele, subQuery) => __awaiter(this, void 0, void 0, function* () {
        const deep = deepRef.current;
        const { data: types = [] } = yield deep.select(Object.assign(Object.assign({}, subQuery), { distinct_on: ['type_id'] }));
        const { data: names = [] } = yield deep.select({
            type_id: deep.idLocal('@deep-foundation/core', 'Contain'),
            to_id: { _in: types.map(l => l.type_id) },
        });
        const menuByTypes = ncy.cxtmenu({
            selector: '.link-node',
            outsideMenuCancel: 10,
            openMenuEvents: 'ctxmenu-travelerMenu-byTypes-open',
            closeMenuEvents: 'ctxmenu-travelerMenu-byTypes-close',
            onClose: () => menuByTypes.destroy(),
            commands: [
                ((query) => ({
                    content: (ele) => { var _a, _b, _c; return `all ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                    select: function (ele) {
                        var _a;
                        return __awaiter(this, void 0, void 0, function* () {
                            const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                            if (id) {
                                yield switchTraveler(query(id), id);
                            }
                        });
                    }
                }))((id) => (Object.assign({}, subQuery))),
                ...types === null || types === void 0 ? void 0 : types.map(l => ((query) => ({
                    content: (ele) => { var _a, _b, _c, _d, _e; return `${((_b = (_a = names.find(n => (n === null || n === void 0 ? void 0 : n.to_id) === l.type_id)) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.value) || l.type_id} ${((_e = findTravlers(query((_c = ele.data('link')) === null || _c === void 0 ? void 0 : _c.id), (_d = ele.data('link')) === null || _d === void 0 ? void 0 : _d.id)) === null || _e === void 0 ? void 0 : _e.length) ? 'x' : '+'}`; },
                    select: function (ele) {
                        var _a;
                        return __awaiter(this, void 0, void 0, function* () {
                            const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                            if (id) {
                                yield switchTraveler(query(id), id);
                            }
                        });
                    }
                }))((id) => (Object.assign(Object.assign({}, subQuery), { type_id: l.type_id })))),
            ],
        });
        ele.emit('ctxmenu-travelerMenu-byTypes-open');
        return {
            destroy: () => {
                ele.emit('ctxmenu-travelerMenu-byTypes-close');
                menuByTypes.destroy();
            },
        };
    });
    const generateMenuByTrees = (ele, subQuery, key) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        const deep = deepRef.current;
        const { data: trees } = yield deep.select({
            distinct_on: ['tree_id'],
            link_id: (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id
        }, { table: 'tree', returning: 'tree_id' });
        const treesIds = trees.map(t => t.tree_id);
        const menuByTypes = ncy.cxtmenu({
            selector: '.link-node',
            outsideMenuCancel: 10,
            openMenuEvents: 'ctxmenu-travelerMenu-byTypes-open',
            closeMenuEvents: 'ctxmenu-travelerMenu-byTypes-close',
            onClose: () => menuByTypes.destroy(),
            commands: [
                ((query) => ({
                    content: (ele) => { var _a, _b, _c; return `all ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                    select: function (ele) {
                        var _a;
                        return __awaiter(this, void 0, void 0, function* () {
                            const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                            if (id) {
                                yield switchTraveler(query(id), id);
                            }
                        });
                    }
                }))((id) => (Object.assign({}, subQuery))),
                ...treesIds === null || treesIds === void 0 ? void 0 : treesIds.map(treeId => ((query) => ({
                    content: (ele) => { var _a, _b, _c; return `${deep.nameLocal(treeId) || 'types'} ${((_c = findTravlers(query((_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id), (_b = ele.data('link')) === null || _b === void 0 ? void 0 : _b.id)) === null || _c === void 0 ? void 0 : _c.length) ? 'x' : '+'}`; },
                    select: function (ele) {
                        var _a;
                        return __awaiter(this, void 0, void 0, function* () {
                            const id = (_a = ele.data('link')) === null || _a === void 0 ? void 0 : _a.id;
                            if (id) {
                                yield switchTraveler(query(id), id);
                            }
                        });
                    }
                }))((id) => {
                    const q = Object.assign({}, subQuery);
                    q[key].tree_id = treeId;
                    return q;
                })),
            ],
        });
        ele.emit('ctxmenu-travelerMenu-byTypes-open');
        return {
            destroy: () => {
                ele.emit('ctxmenu-travelerMenu-byTypes-close');
                menuByTypes.destroy();
            },
        };
    });
    return {
        destroy: () => {
            nodeMenuTraveler.destroy();
        },
        insertTraveler,
        deleteTraveler,
        switchTraveler,
        findTravlers,
    };
}
//# sourceMappingURL=traveler.js.map