import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Box, Center, Flex } from '@chakra-ui/react';
import { motion, useMotionValue, useTransform } from 'framer-motion';
import React, { useEffect, useRef } from 'react';
import ReactResizeDetector from 'react-resize-detector';
export const EditorGrid = React.memo(({ columns = 'repeat(2, 50%)', editorTabsElement, editorTextAreaElement, editorRight, closeAllButtonElement, closeButtonElement, editorRightSwitch, sash = false, heightEditorGrid = '100vh', }) => {
    return (_jsxs(Box, { display: 'flex', flexDir: 'column', h: heightEditorGrid, overflowY: 'hidden', sx: { opacity: 0.98 }, children: [editorTabsElement
                ? _jsxs(Flex, { children: [_jsx(Center, { children: closeAllButtonElement }), _jsx(Box, { sx: { width: 'calc(100% - 4rem)' }, children: editorTabsElement }), _jsx(Center, { children: closeButtonElement })] })
                : null, sash
                ? (_jsx(VerticalSash, { editorTextAreaElement: editorTextAreaElement, editorRight: editorRight, editorRightSwitch: editorRightSwitch }))
                : (_jsxs(Box, { display: 'grid', gridTemplateColumns: columns, bg: 'backgroundModal', height: '100%', children: [editorTextAreaElement, _jsxs(Box, { display: 'grid', gridTemplateRows: '1fr max-content', h: '100%', position: "relative", overflowY: 'scroll', children: [editorRight, editorRightSwitch] })] }))] }));
});
export const VerticalSash = ({ editorTextAreaElement, editorRight, editorRightSwitch, }) => {
    const sashX = useMotionValue(0);
    const leftWidth = useTransform(sashX, (value) => `${Math.max(0, value)}px`);
    const rightPaneLeft = useTransform(sashX, (value) => `${Math.max(0, value) + 5}px`);
    const sashRef = useRef(null);
    const containerRef = useRef(null);
    const rightPaneRef = useRef(null);
    const isDragging = useRef(false);
    const isBrowser = () => typeof window !== undefined;
    useEffect(() => {
        const updateRightPanelWidth = () => {
            if (rightPaneRef.current) {
                const leftPaneWidth = parseFloat(leftWidth.get());
                const container = containerRef.current;
                const rightPane = rightPaneRef.current;
                const containerWidth = container.offsetWidth;
                const rightPaneWidth = containerWidth - leftPaneWidth - 5;
                rightPane.style.width = `${rightPaneWidth}px`;
            }
        };
        leftWidth.onChange(updateRightPanelWidth);
        window.addEventListener('resize', updateRightPanelWidth);
        const handleMove = (clientX) => {
            if (isDragging.current && sashRef.current) {
                const container = containerRef.current;
                const containerRect = container.getBoundingClientRect();
                sashX.set(clientX - containerRect.left);
            }
        };
        const handleMouseMove = (event) => {
            handleMove(event.clientX);
        };
        const handleTouchMove = (event) => {
            handleMove(event.touches[0].clientX);
        };
        const handleMouseUp = () => {
            isDragging.current = false;
        };
        const handleTouchEnd = () => {
            isDragging.current = false;
        };
        if (isBrowser()) {
            window.addEventListener('mousemove', handleMouseMove);
            window.addEventListener('mouseup', handleMouseUp);
            window.addEventListener('touchmove', handleTouchMove);
            window.addEventListener('touchend', handleTouchEnd);
            window.removeEventListener('resize', updateRightPanelWidth);
            return () => {
                window.removeEventListener('mousemove', handleMouseMove);
                window.removeEventListener('mouseup', handleMouseUp);
                window.removeEventListener('touchmove', handleTouchMove);
                window.removeEventListener('touchend', handleTouchEnd);
            };
        }
    }, [leftWidth, sashX]);
    const onResize = (width) => {
        sashX.set(width / 2);
    };
    return (_jsx(ReactResizeDetector, { handleWidth: true, onResize: onResize, targetRef: containerRef, handleHeight: false, children: _jsxs("div", { ref: containerRef, style: { position: 'relative', width: '100%', height: '100%' }, children: [_jsx(motion.div, { style: {
                        position: 'absolute',
                        width: leftWidth,
                        height: '100%',
                        borderRight: '1px solid #ccc',
                        overflow: 'initial',
                    }, children: editorTextAreaElement }), _jsx(motion.div, { ref: sashRef, style: {
                        position: 'absolute',
                        left: leftWidth,
                        width: '5px',
                        height: '100%',
                        background: '#ccc',
                        cursor: 'col-resize',
                    }, onPointerDown: () => {
                        isDragging.current = true;
                    } }), _jsx(motion.div, { ref: rightPaneRef, style: {
                        position: 'absolute',
                        left: rightPaneLeft,
                        right: 0,
                        height: '100%',
                        overflow: 'auto',
                    }, children: _jsxs(Box, { display: 'grid', gridTemplateRows: '1fr max-content', h: '100%', position: "relative", overflowY: 'scroll', children: [editorRight, editorRightSwitch] }) })] }) }));
};
//# sourceMappingURL=editor-grid.js.map