import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Box, Button, Text } from '@chakra-ui/react';
import { useDebounceCallback } from '@react-hook/debounce';
import { AnimatePresence, useCycle, motion, LazyMotion, domAnimation } from 'framer-motion';
import React, { useCallback, useEffect, useRef, useState } from 'react';
import { useHotkeys } from 'react-hotkeys-hook';
import { TbArrowRotaryFirstRight } from 'react-icons/tb';
const sideVariants = {
    closed: {
        transition: {
            staggerChildren: 0.1,
            staggerDirection: 1
        }
    },
    open: {
        y: "-11.2rem",
        transition: {
            staggerChildren: 0.1,
            staggerDirection: 1
        }
    },
    initial: {
        originY: 0,
        originX: 0,
    }
};
const itemVariants = {
    closed: {
        opacity: 0
    },
    open: { opacity: 1 }
};
const iconVariants = {
    closed: {
        rotate: 0,
        transition: {
            type: "tween",
            duration: 0.2,
            delay: 0.7
        }
    },
    open: {
        rotate: 180,
        transition: {
            type: "tween",
            duration: 0.2
        }
    }
};
export const ListLanguages = React.memo(({ languages = [], currentLanguage, setLanguage, }) => {
    const [open, cycleOpen] = useCycle(false, true);
    const languagesListRef = useRef(null);
    const buttonRef = useRef(null);
    const [searchString, setSearchString] = useState('');
    const debouncedSearch = useDebounceCallback((search) => {
        var _a, _b;
        for (let i = 0; i < languages.length; i++) {
            const l = languages[i];
            if (l.id.toLowerCase().startsWith(search)) {
                (_b = (_a = languagesListRef.current) === null || _a === void 0 ? void 0 : _a.children[i]) === null || _b === void 0 ? void 0 : _b.focus();
            }
        }
    }, 300);
    const handleKeyDown = (e) => {
        if (open == true && /^[a-zA-Z]$/.test(e.key)) {
            e.preventDefault();
            setSearchString(searchString + e.key.toLowerCase());
        }
        else if (open && e.key === 'Backspace') {
            setSearchString(searchString.slice(0, -1));
        }
    };
    useEffect(() => {
        debouncedSearch(searchString);
    }, [searchString, debouncedSearch]);
    useEffect(() => {
        window.addEventListener('keydown', handleKeyDown);
        return () => {
            window.removeEventListener('keydown', handleKeyDown);
        };
    }, [open, languages, searchString, handleKeyDown]);
    const selectLanguage = useCallback((l) => {
        setSearchString((prevLang) => prevLang == l ? 0 : l);
    }, []);
    useHotkeys('enter', e => {
        var _a;
        e.preventDefault();
        e.stopPropagation();
        if (searchString) {
            setLanguage((_a = e.srcElement) === null || _a === void 0 ? void 0 : _a.computedName);
            cycleOpen();
            setSearchString('');
        }
        return console.log('event', e);
    }, { enableOnFormTags: ["SELECT", "INPUT"] });
    return (_jsxs(LazyMotion, { features: domAnimation, children: [_jsx(Box, { position: 'relative', sx: { height: 0, width: '100%' }, children: _jsx(AnimatePresence, { children: open && (_jsx(Box, { as: motion.div, animate: {
                            scale: 1,
                            transition: { duration: 0.3, type: 'spring' }
                        }, exit: {
                            scale: 0,
                            y: '2rem',
                            transition: { delay: 0.7, duration: 0.3, type: 'spring' }
                        }, sx: {
                            height: '2rem',
                            width: 'max-content',
                            top: 0,
                            left: 0,
                            position: 'absolute'
                        }, children: _jsx(Box, { as: motion.ul, initial: 'initial', animate: 'open', exit: 'closed', ref: languagesListRef, variants: sideVariants, sx: {
                                borderRadius: '0.5rem',
                                position: 'relative',
                                background: 'bgColor',
                                listStyle: 'none',
                                padding: '0.5rem',
                                height: '11rem',
                                overflowY: 'scroll',
                                overscrollBehavior: 'contain',
                                filter: 'drop-shadow(0px 0px 1px #5f6977)',
                                outline: `solid 4px`,
                                outlineColor: 'bgColor',
                                outlineOffset: '-4px',
                                '&>*:not(:last-child)': {
                                    pt: '0.2rem',
                                    pb: '0.2rem',
                                }
                            }, children: languages && languages.map(l => (_jsx(Box, { as: motion.li, role: 'button', tabIndex: 0, whileHover: { scale: 1.1, originX: 0 }, variants: itemVariants, onClick: () => {
                                    setLanguage(l.id);
                                    cycleOpen();
                                    setSearchString('');
                                }, _focus: {
                                    p: '0.2rem 0.5rem',
                                }, children: _jsx(Text, { fontSize: 'sm', children: l.id }) }, l.id))) }) })) }) }), _jsx(Box, { position: 'relative', sx: {
                    height: '2rem',
                    width: 'max-content',
                }, children: _jsx(Box, { position: 'absolute', children: _jsx(Button, { ref: buttonRef, as: motion.button, whileTap: { scale: 0.97 }, bg: open == true ? 'handlersInput' : 'bgLanguagesButton', onClick: () => cycleOpen(), sx: {
                            height: '2rem',
                            width: '11.2rem',
                            filter: 'drop-shadow(0px 0px 1px #5f6977)',
                            _hover: {
                                bg: 'handlersInput'
                            },
                        }, rightIcon: _jsx(Box, { as: motion.div, variants: iconVariants, animate: open ? 'open' : 'closed', children: _jsx(TbArrowRotaryFirstRight, {}) }), children: _jsx(Text, { fontSize: 'sm', children: currentLanguage }) }) }) })] }));
});
//# sourceMappingURL=editor-lang-list.js.map