import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Box, CloseButton as _CloseButton, Flex, IconButton, Spinner, useColorMode } from '@chakra-ui/react';
import { AnimatePresence, Reorder } from 'framer-motion';
import React, { useState } from 'react';
import { isAndroid, isIOS } from 'react-device-detect';
import { VscChromeClose } from 'react-icons/vsc';
import { useChackraColor } from '../get-color';
;
export const EditorTabs_ = React.memo(({ tabs = [], onClick, onClose, }) => {
    const gray900 = useChackraColor('gray.900');
    const white = useChackraColor('white');
    const { colorMode } = useColorMode();
    return (_jsx(Flex, { position: "sticky", top: "0", bg: colorMode == 'light' ? white : gray900, zIndex: "sticky", height: "max-content", alignItems: "center", flexWrap: "nowrap", overflowX: "auto", borderBottomStyle: 'solid', borderBottomWidth: 1, borderBottomColor: colorMode == 'light' ? 'blackAlpha.200' : 'whiteAlpha.200', as: "nav", css: {
            WebkitOverflowScrolling: "touch",
            msOverflowStyle: "-ms-autohiding-scrollbar"
        }, children: _jsx(Flex, { flex: "0 0 auto", children: tabs.map(t => _jsx(EditorTab, { tab: t, onClick: onClick, onClose: onClose }, t.id)) }) }));
});
export const NonSavedIcon = React.memo(({ bg = 'primary', borderColor = 'red.200', }) => {
    return (_jsx(Box, { w: '0.5rem', h: '0.5rem', bg: bg, borderStyle: 'solid', borderWidth: 1, borderColor: borderColor, borderRadius: 'full' }));
});
export const CloseButton = React.memo(({ onClick, }) => {
    const gray900 = useChackraColor('gray.900');
    const white = useChackraColor('white');
    const { colorMode } = useColorMode();
    return _jsx(_CloseButton, { size: 'md', bg: colorMode == 'light' ? white : gray900, borderRadius: 'none', height: '100%', borderStyle: 'solid', borderWidth: 1, borderColor: colorMode == 'light' ? 'blackAlpha.200' : 'whiteAlpha.200', onClick: onClick });
});
export const EditorTab = React.memo(({ tab, onClick, onSaveTab, onClose, }) => {
    const { id, title, saved = false, loading = false, active = false, } = tab;
    const gray900 = useChackraColor('gray.900');
    const white = useChackraColor('white');
    const { colorMode } = useColorMode();
    return (_jsx(Reorder.Item, { value: tab, as: 'div', id: `tab-${id}`, initial: { opacity: 0, y: 30 }, animate: {
            opacity: 1,
            y: 0,
            transition: { duration: 0.15 }
        }, exit: { opacity: 0, y: 20, transition: { duration: 0.3 } }, className: 'tab', onPointerDown: () => onClick && onClick(tab), children: _jsxs(Box, { as: 'button', "aria-label": `tab-${id}`, display: 'flex', alignItems: 'center', w: 'max', h: 'max', lineHeight: '1.4', transition: 'all 0.2s cubic-bezier(.08,.52,.52,1)', pl: '3', pr: '2', py: '2', borderRadius: 'none', fontSize: 'xs', fontWeight: 'normal', bg: active ? 'handlersInput' : 'bgLanguagesButton', borderRightWidth: 'thin', borderRightColor: 'borderColor', _hover: { bg: active ? 'handlersInput' : 'whiteGray' }, _active: {
                transform: 'scale(0.98)',
            }, children: [_jsx(Box, { flex: '1', mr: '2', children: title }), isIOS || isAndroid
                    ? _jsx(IconButton, { colorScheme: 'gray', isRound: true, size: 'xs', onClick: () => onSaveTab && onSaveTab(tab), "aria-label": 'Save button', icon: !saved && _jsx(NonSavedIcon, {}) || loading && _jsx(Spinner, { size: 'xs' }) })
                    : _jsx(Box, { mr: '3', children: !saved && _jsx(NonSavedIcon, {}) || loading && _jsx(Spinner, { size: 'xs' }) }), _jsx(Box, { onClick: (e) => {
                        e === null || e === void 0 ? void 0 : e.stopPropagation();
                        onClose && onClose(tab);
                    }, as: 'button', "aria-label": 'close tab button', borderRadius: 'md', p: 1, _hover: {
                        bg: colorMode == 'light' ? 'blackAlpha.100' : 'gray.700',
                    }, children: _jsx(VscChromeClose, {}) })] }) }));
});
export const EditorTabs = React.memo(({ tabs = [], setTabs, onClick, onClose, onSaveTab, }) => {
    const gray900 = useChackraColor('gray.900');
    const white = useChackraColor('white');
    const { colorMode } = useColorMode();
    const [selectedTab, setSelectedTab] = useState(tabs[0]);
    return (_jsx(Reorder.Group, { as: 'div', axis: "x", onReorder: setTabs, style: {
            display: 'flex',
            position: "sticky",
            top: 0,
            zIndex: "sticky",
            height: "max-content",
            alignItems: "center",
            flexWrap: "nowrap",
            overflowX: "auto",
            borderBottomWidth: 'thin',
            borderBottomColor: colorMode == 'light' ? 'blackAlpha.200' : 'whiteAlpha.200',
            WebkitOverflowScrolling: "touch",
            msOverflowStyle: "-ms-autohiding-scrollbar",
        }, values: tabs, children: _jsx(AnimatePresence, { initial: false, children: tabs.map(t => _jsx(EditorTab, { tab: t, onClick: onClick, onSaveTab: onSaveTab, onClose: onClose }, t)) }) }));
});
//# sourceMappingURL=editor-tabs.js.map