import { jsx as _jsx } from "react/jsx-runtime";
import { useColorMode } from '@chakra-ui/react';
import dynamic from 'next/dynamic';
import React from 'react';
import _ from 'lodash';
const MonacoEditor = dynamic(() => import('@monaco-editor/react').then(m => m.default), { ssr: false });
const monacoEditorOptions = {
    wordWrap: true,
};
export const EditorTextArea = React.memo(({ refEditor = { current: undefined }, value, onChange, onSave, onClose, onExit, minimap = true, lineNumbers = 'on', defaultLanguage = "javascript", onMount, }) => {
    const refValue = React.useRef(value);
    refValue.current = value;
    const { colorMode } = useColorMode();
    const handleEditorDidMount = (editor, monaco) => {
        refEditor.current = { editor, monaco };
        editor.getModel().updateOptions({ tabSize: 2 });
        editor.addCommand(monaco.KeyMod.CtrlCmd | monaco.KeyCode.KeyS, () => {
            onSave && onSave(refValue.current);
        });
        editor.addCommand(monaco.KeyMod.CtrlCmd | monaco.KeyCode.KeyE, () => {
            onClose && onClose();
        });
        editor.addCommand(monaco.KeyMod.CtrlCmd | monaco.KeyCode.Escape, () => {
            onExit && onExit();
        });
        onMount && onMount(editor, monaco);
    };
    return (_jsx(MonacoEditor, { options: Object.assign(Object.assign({}, monacoEditorOptions), { minimap: {
                enabled: minimap
            }, lineNumbers: lineNumbers }), height: "100%", width: "100%", theme: colorMode === 'light' ? 'light' : "vs-dark", defaultLanguage: defaultLanguage, defaultValue: _.toString(value) || '', onChange: onChange, onMount: handleEditorDidMount }));
});
//# sourceMappingURL=editor-textarea.js.map