import { Fragment as _Fragment, jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useQuery, useSubscription } from "@apollo/client";
import { useDeep, useDeepId } from "@deep-foundation/deeplinks/imports/client";
import { generateQuery, generateQueryData } from "@deep-foundation/deeplinks/imports/gql";
import { useMinilinksFilter } from "@deep-foundation/deeplinks/imports/minilinks";
import { memo, useCallback, useEffect, useMemo, useRef, useState } from "react";
import { useBreadcrumbs } from "./hooks";
import { useDelayedInterval } from "./use-delayed-interval";
export function DeepLoaderActive({ name, query: queryLink, onChange, debounce = 1000, subscription = true, }) {
    var _a, _b;
    const useApolloLoader = subscription ? useSubscription : useQuery;
    const deep = useDeep();
    const subQuery = useMemo(() => {
        var _a;
        const v = ((_a = queryLink === null || queryLink === void 0 ? void 0 : queryLink.value) === null || _a === void 0 ? void 0 : _a.value);
        const variables = deep.serializeQuery(v);
        const where = variables === null || variables === void 0 ? void 0 : variables.where;
        return generateQuery({
            operation: subscription ? 'subscription' : 'query',
            queries: [generateQueryData({
                    tableName: 'links',
                    returning: `
          id type_id from_id to_id value string { id value } number { id value } object { id value }
        `,
                    variables: v
                        ? Object.assign(Object.assign({}, variables), { where }) : { where: {}, limit: 0 },
                })],
            name: name,
        });
    }, [queryLink, (_a = queryLink === null || queryLink === void 0 ? void 0 : queryLink.value) === null || _a === void 0 ? void 0 : _a.value]);
    const subQueryResults = useApolloLoader(subQuery.query, { variables: subQuery.variables });
    const [sintSubQueryResults, setSintSubQueryResults] = useState(subQueryResults);
    const subQueryPrimary = subscription ? subQueryResults : sintSubQueryResults || subQueryResults;
    const delayedSubQueryRef = useRef();
    delayedSubQueryRef.current = subQuery;
    useDelayedInterval(useCallback(() => new Promise((res, rej) => {
        if (subscription)
            rej();
        else
            subQueryResults === null || subQueryResults === void 0 ? void 0 : subQueryResults.refetch(delayedSubQueryRef.current.variables).then((r) => {
                setSintSubQueryResults(r);
                res(undefined);
            });
    }), [queryLink, (_b = queryLink === null || queryLink === void 0 ? void 0 : queryLink.value) === null || _b === void 0 ? void 0 : _b.value]), debounce);
    useEffect(() => {
        var _a, _b;
        if (((_a = subQueryPrimary === null || subQueryPrimary === void 0 ? void 0 : subQueryPrimary.data) === null || _a === void 0 ? void 0 : _a.q0) && !(subQueryPrimary === null || subQueryPrimary === void 0 ? void 0 : subQueryPrimary.loading))
            onChange && onChange((_b = subQueryPrimary === null || subQueryPrimary === void 0 ? void 0 : subQueryPrimary.data) === null || _b === void 0 ? void 0 : _b.q0);
    }, [subQueryPrimary]);
    useEffect(() => {
        return () => {
            onChange && onChange([]);
        };
    }, []);
    return _jsx(_Fragment, {});
}
export const DeepLoader = memo(function DeepLoader({ spaceId, }) {
    const deep = useDeep();
    const userId = deep.linkId;
    const [breadcrumbs] = useBreadcrumbs();
    const { data: HandleCyto } = useDeepId('@deep-foundation/handle-cyto', 'HandleCyto');
    const spaceQuery = useMemo(() => ({ value: { value: Object.assign(Object.assign({ up: {
                    tree_id: { _eq: deep.idLocal('@deep-foundation/core', 'containTree') },
                    parent_id: { _eq: spaceId },
                } }, ({
                type_id: {
                    _nin: [
                        deep.idLocal('@deep-foundation/core', 'Then'),
                        deep.idLocal('@deep-foundation/core', 'Promise'),
                        deep.idLocal('@deep-foundation/core', 'Resolved'),
                        deep.idLocal('@deep-foundation/core', 'Rejected'),
                        deep.idLocal('@deep-foundation/core', 'PromiseResult'),
                    ],
                },
            })), { _not: {
                    _and: [
                        {
                            _not: {
                                in: {
                                    type_id: deep.idLocal('@deep-foundation/core', 'Contain'),
                                    from_id: { _eq: spaceId }
                                }
                            }
                        },
                        {
                            up: {
                                tree_id: { _eq: deep.idLocal('@deep-foundation/core', 'containTree') },
                                self: { _eq: false },
                                parent: {
                                    type_id: { _in: [deep.idLocal('@deep-foundation/core', 'Space'), deep.idLocal('@deep-foundation/core', 'Package')] },
                                    up: {
                                        tree_id: { _eq: deep.idLocal('@deep-foundation/core', 'containTree') },
                                        self: { _eq: false },
                                        parent_id: { _eq: spaceId },
                                    },
                                },
                            },
                        }
                    ]
                } }) } }), []);
    const breadcrumbsQuery = useMemo(() => ({ value: { value: {
                down: {
                    tree_id: { _eq: deep.idLocal('@deep-foundation/core', 'containTree') },
                    link_id: { _eq: spaceId },
                },
            } } }), []);
    let queries = useMinilinksFilter(deep.minilinks, useCallback((l, ...args) => {
        return [deep.idLocal('@deep-foundation/core', 'Query'), deep.idLocal('@deep-foundation/core', 'Active')].includes(l.type_id);
    }, [spaceId]), useCallback((l, ml) => {
        var _a;
        return (_a = ml.byType[deep.idLocal('@deep-foundation/core', 'Query')]) === null || _a === void 0 ? void 0 : _a.filter((l) => {
            var _a, _b, _c;
            return (l === null || l === void 0 ? void 0 : l.type_id) === deep.idLocal('@deep-foundation/core', 'Query') && !!((_b = (_a = l === null || l === void 0 ? void 0 : l.inByType) === null || _a === void 0 ? void 0 : _a[deep.idLocal('@deep-foundation/core', 'Active')]) === null || _b === void 0 ? void 0 : _b.find(a => (a === null || a === void 0 ? void 0 : a.from_id) === spaceId)) && ((_c = l === null || l === void 0 ? void 0 : l.value) === null || _c === void 0 ? void 0 : _c.value);
        });
    }, [spaceId]), 1000);
    queries = queries || [];
    const insertableTypesQuery = useMemo(() => ({ value: { value: {
                _or: [
                    {
                        can_object: {
                            action_id: { _eq: 121 },
                            subject_id: { _eq: userId }
                        }
                    }, {
                        type_id: { _in: [
                                deep.idLocal('@deep-foundation/core', 'Type'),
                                deep.idLocal('@deep-foundation/core', 'HandleOperation'),
                                deep.idLocal('@deep-foundation/core', 'Operation'),
                                deep.idLocal('@deep-foundation/core', 'TreeInclude'),
                                deep.idLocal('@deep-foundation/core', 'File'),
                            ] }
                    },
                ],
            } } }), [userId]);
    const notLoadedEnds = useMinilinksFilter(deep.minilinks, useCallback((l) => { var _a; return !((_a = l === null || l === void 0 ? void 0 : l._applies) === null || _a === void 0 ? void 0 : _a.includes('not-loaded-ends')); }, []), useCallback((l, ml) => (ml.links.filter(l => ((!l._applies.includes('contains_and_symbols')
        &&
            !!l._applies.find((a) => !!~a.indexOf('query-') || a === 'space' || a === 'breadcrumbs')) && ((!!l.from_id && (!l.from || !!l.from._applies.includes('not-loaded-ends')))
        ||
            (!!l.to_id && (!l.to || !l.to._applies.includes('not-loaded-ends'))))))), []), 1000) || [];
    const notLoadedEndsQuery = useMemo(() => {
        return { value: { value: {
                    id: { _in: [
                            ...notLoadedEnds.map(l => l.from_id).filter(id => !!id),
                            ...notLoadedEnds.map(l => l.to_id).filter(id => !!id),
                        ] },
                } } };
    }, [notLoadedEnds]);
    const treesQuery = useMemo(() => {
        return { value: { value: {
                    type_id: deep.idLocal('@deep-foundation/core', 'Tree'),
                } } };
    }, [notLoadedEnds]);
    const typeIds = useMinilinksFilter(deep.minilinks, useCallback((l) => true, []), useCallback((l, ml) => {
        return Object.keys(ml.byType).map(type => parseInt(type));
    }, []), 1000) || [];
    const treeIds = useMinilinksFilter(deep.minilinks, useCallback((l) => true, []), useCallback((l, ml) => {
        var _a, _b;
        return ((_b = (_a = ml.byType) === null || _a === void 0 ? void 0 : _a[deep.idLocal('@deep-foundation/core', 'Tree')]) === null || _b === void 0 ? void 0 : _b.map(l => l.id)) || [];
    }, []), 1000) || [];
    const ids = useMinilinksFilter(deep.minilinks, useCallback((l) => true, []), useCallback((l, ml) => {
        return Object.keys(ml.byId).map(link => parseInt(link));
    }, []), 1000) || [];
    const queryAndSpaceLoadedIds = useMinilinksFilter(deep.minilinks, useCallback((l) => { var _a; return !!((_a = l === null || l === void 0 ? void 0 : l._applies) === null || _a === void 0 ? void 0 : _a.find(a => a.includes('query-') || a.includes('space'))); }, []), useCallback((l, ml) => (ml.links.filter(l => { var _a; return (_a = l._applies) === null || _a === void 0 ? void 0 : _a.find(a => a.includes('query-') || a.includes('space') || a.includes('client-handlers')); }).map(l => l.id)), []), 1000) || [];
    const containsAndSymbolsQuery = useMemo(() => {
        const ids = [...typeIds, ...queryAndSpaceLoadedIds, ...treeIds];
        return { value: { value: {
                    to_id: { _in: ids },
                    type_id: { _in: [deep.idLocal('@deep-foundation/core', 'Contain'), deep.idLocal('@deep-foundation/core', 'Symbol')] },
                } } };
    }, [typeIds, queryAndSpaceLoadedIds, treeIds]);
    const valuesQuery = useMemo(() => {
        const ids = [...typeIds, ...queryAndSpaceLoadedIds];
        return { value: { value: {
                    from_id: { _in: ids },
                    type_id: { _eq: deep.idLocal('@deep-foundation/core', 'Value') },
                } } };
    }, [typeIds, queryAndSpaceLoadedIds]);
    const typesQuery = useMemo(() => {
        return { value: { value: {
                    down: {
                        tree_id: { _eq: deep.idLocal('@deep-foundation/core', 'typesTree') },
                        link_id: { _in: ids }
                    },
                } } };
    }, [ids]);
    const clientHandlersQuery = useMemo(() => {
        const _ids = [...ids, ...queryAndSpaceLoadedIds];
        const handles = { _in: [deep.idLocal('@deep-foundation/core', 'HandleClient'), HandleCyto] };
        return { value: { value: {
                    _or: [
                        {
                            in: {
                                type_id: deep.idLocal('@deep-foundation/core', 'Handler'),
                                in: {
                                    type_id: handles,
                                },
                            },
                        },
                        {
                            type_id: deep.idLocal('@deep-foundation/core', 'Handler'),
                            in: {
                                type_id: handles,
                            },
                        },
                        { type_id: handles },
                    ]
                } } };
    }, [queryAndSpaceLoadedIds, ids]);
    const handleCytoCode = useMemo(() => {
        const _ids = [...ids, ...queryAndSpaceLoadedIds];
        return { value: { value: {
                    _or: [
                        {
                            type_id: deep.idLocal('@deep-foundation/core', 'Handler'),
                            in: {
                                type_id: { _in: deep.idLocal('@deep-foundation/core', 'HandleClient'), HandleCyto },
                            },
                        },
                        { type_id: deep.idLocal('@deep-foundation/core', 'HandleClient') },
                    ]
                } } };
    }, [queryAndSpaceLoadedIds, ids]);
    return _jsxs(_Fragment, { children: [_jsx(_Fragment, { children: _jsx(DeepLoaderActive, { name: "DEEPCASE_SPACE", query: spaceQuery, onChange: (r) => {
                        var _a;
                        (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.apply(r, 'space');
                    } }, "DEEPCASE_SPACE") }), _jsx(_Fragment, { children: _jsx(DeepLoaderActive, { name: "DEEPCASE_TREES", query: treesQuery, onChange: (r) => {
                        var _a;
                        (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.apply(r, 'trees');
                    } }, "DEEPCASE_TREES") }), _jsx(_Fragment, { children: _jsx(DeepLoaderActive, { name: "DEEPCASE_NOT_LOADED_ENDS", query: notLoadedEndsQuery, onChange: (r) => {
                        var _a;
                        (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.apply(r, 'not-loaded-ends');
                    } }, "DEEPCASE_NOT_LOADED_ENDS") }), !!breadcrumbs && _jsx(_Fragment, { children: _jsx(DeepLoaderActive, { name: "DEEPCASE_BREADCRUMBS", query: breadcrumbsQuery, onChange: (r) => {
                        var _a;
                        (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.apply(r, 'breadcrumbs');
                    } }, "DEEPCASE_BREADCRUMBS") }), _jsx(_Fragment, { children: _jsx(DeepLoaderActive, { name: "DEEPCASE_CLIENT_HANDLERS", query: clientHandlersQuery, onChange: (r) => {
                        var _a;
                        (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.apply(r, 'client-handlers');
                    } }, "DEEPCASE_CLIENT_HANDLERS") }), queries === null || queries === void 0 ? void 0 : queries.map((f, i) => (_jsx(DeepLoaderActive, { name: `DEEPCASE_QUERY_${f.id}`, query: f, onChange: (r) => {
                    var _a;
                    (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.apply(r, `query-${f.id}`);
                } }, `DEEPCASE_QUERY_${f.id}`))), _jsx(_Fragment, { children: _jsx(DeepLoaderActive, { name: `DEEPCASE_TYPES`, query: typesQuery, onChange: (r) => {
                        var _a;
                        (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.apply(r, 'types');
                    } }, `DEEPCASE_TYPES`) }), !!typeIds && _jsx(_Fragment, { children: _jsx(DeepLoaderActive, { name: `DEEPCASE_CONTAINS_AND_SYMBOLS`, query: containsAndSymbolsQuery, debounce: 2000, onChange: (r) => {
                        var _a;
                        (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.apply(r, 'contains_and_symbols');
                    } }, `DEEPCASE_CONTAINS_AND_SYMBOLS`) }), !!typeIds && _jsx(_Fragment, { children: _jsx(DeepLoaderActive, { name: `DEEPCASE_VALUES`, query: valuesQuery, onChange: (r) => {
                        var _a;
                        (_a = deep.minilinks) === null || _a === void 0 ? void 0 : _a.apply(r, 'values');
                    } }, `DEEPCASE_VALUES`) })] });
});
//# sourceMappingURL=loader.js.map